@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland <vadim@fwbuilder.org>
 
-  $Id: Address.cpp 247 2009-01-24 07:16:47Z vadim $
+  $Id: Address.cpp 473 2009-11-24 21:12:55Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -165,10 +165,10 @@ bool Address::belongs(const InetAddr &other) const
     return false;
 }
 
-bool Address::cmp(const FWObject *obj) throw(FWException)
+bool Address::cmp(const FWObject *obj, bool recursive) throw(FWException)
 {
     if (Address::constcast(obj)==NULL) return false;
-    if (!FWObject::cmp(obj)) return false;
+    if (!FWObject::cmp(obj, recursive)) return false;
     if (hasInetAddress()!=Address::constcast(obj)->hasInetAddress()) return false;
     if (!hasInetAddress()) return true;
     // both this and obj are objects that own ip addresses
