@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: CustomService.cpp 238 2008-12-29 17:49:04Z vadim $
+  $Id: CustomService.cpp 614 2010-06-13 04:59:57Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -38,13 +38,13 @@ using namespace libfwbuilder;
 
 const char *CustomService::TYPENAME={"CustomService"};
 
-CustomService::CustomService() {}
+CustomService::CustomService() { address_family = AF_INET;}
 CustomService::CustomService(const FWObjectDatabase *root,bool prepopulate) :
-    Service(root, prepopulate) {}
+    Service(root, prepopulate) { address_family = AF_INET;}
 CustomService::~CustomService() {}
 
 string CustomService::getProtocolName()   { return getProtocol(); }
-int    CustomService::getProtocolNumber() { return 65000; }
+int    CustomService::getProtocolNumber() const { return 65000; }
 
 
 FWObject& CustomService::shallowDuplicate(const FWObject *x,
@@ -102,17 +102,17 @@ void CustomService::fromXML(xmlNodePtr root) throw(FWException)
 
     for (xmlNodePtr cur=root->xmlChildrenNode; cur; cur=cur->next)
     {
-        if(cur && !xmlIsBlankNode(cur))
+        if (cur && !xmlIsBlankNode(cur))
         {
-    	    n=FROMXMLCAST(xmlGetProp(cur,TOXMLCAST("platform")));
-	        assert(n!=NULL);
-	        cont=FROMXMLCAST( xmlNodeGetContent(cur) );
-	        if (cont)
+    	    n = FROMXMLCAST(xmlGetProp(cur,TOXMLCAST("platform")));
+            assert(n!=NULL);
+            cont = FROMXMLCAST( xmlNodeGetContent(cur) );
+            if (cont)
             {
-		        setCodeForPlatform(n, cont );
-                FREEXMLBUFF(n);
+                setCodeForPlatform(n, cont );
                 FREEXMLBUFF(cont);
             }
+            FREEXMLBUFF(n);
         }
     }
 }
@@ -146,10 +146,10 @@ xmlNodePtr CustomService::toXML(xmlNodePtr parent) throw(FWException)
     return me;
 }
 
-bool CustomService::cmp(const FWObject *obj) throw(FWException)
+bool CustomService::cmp(const FWObject *obj, bool recursive) throw(FWException)
 {
     if (CustomService::constcast(obj)==NULL) return false;
-    if (!FWObject::cmp(obj)) return false;
+    if (!FWObject::cmp(obj, recursive)) return false;
     
     const CustomService *o2 = CustomService::constcast(obj);
 
