@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: DNSName.cpp 246 2009-01-23 21:36:41Z vadim $
+  $Id: DNSName.cpp 574 2010-03-04 01:19:59Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -116,7 +116,7 @@ xmlNodePtr DNSName::toXML(xmlNodePtr parent) throw(FWException)
 }
 
 
-void DNSName::loadFromSource(bool ipv6) throw(FWException)
+void DNSName::loadFromSource(bool ipv6, bool test_mode) throw(FWException)
 {
     int af_type = (ipv6)?AF_INET6:AF_INET;
     try
@@ -126,14 +126,19 @@ void DNSName::loadFromSource(bool ipv6) throw(FWException)
         {
             //Address *a = Address::cast(
             //    getRoot()->create((ipv6)?IPv6::TYPENAME:IPv4::TYPENAME));
+            int af = AF_INET;
             Address *a = NULL;
-            if (ipv6) a = getRoot()->createIPv6();
+            if (ipv6) { a = getRoot()->createIPv6(); af = AF_INET6; }
             else a = getRoot()->createIPv4();
-            a->setAddress( *i );
+            a->setAddress(*i);
+            a->setNetmask(InetAddr::getAllOnes(af));
             addRef(a);
         }
     } catch (const FWException &ex)
     {
+        // in test mode we use dummy address but still throw exception.
+        // Compiler should print error message but continue.
+
         ostringstream err;
         string af_type_name = (ipv6)?string("AF_INET6"):string("AF_INET");
 
@@ -144,8 +149,29 @@ void DNSName::loadFromSource(bool ipv6) throw(FWException)
             << "\" "
             << "(" << af_type_name << ")"
             << ": "
-            << ex.toString()
-            << endl;
+            << ex.toString();
+
+        if (test_mode)
+        {
+            err << " Using dummy address in test mode";
+            int af = AF_INET;
+            Address *a = NULL;
+            if (ipv6)
+            {
+                a = getRoot()->createIPv6();
+                a->setAddress(InetAddr(af_type, "2001:db8::1"));
+                af = AF_INET6;
+            } else
+            {
+                a = getRoot()->createIPv4();
+                a->setAddress("192.0.2.1");
+                a->setNetmask(InetAddr::getAllOnes(af));
+            }
+            addRef(a);
+            a->setBool(".rule_error", true);
+            a->setStr(".error_msg", err.str());
+        }
+
         throw(FWException(err.str()));
     }
 }
