@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWObject.cpp 268 2009-04-21 03:05:51Z vadim $
+  $Id: FWObject.cpp 606 2010-05-23 22:50:18Z a2k $
 
 
   This program is free software which we release under the GNU General Public
@@ -38,6 +38,7 @@
 #include <fwbuilder/RuleElement.h>
 #include <fwbuilder/Rule.h>
 #include <fwbuilder/Host.h>
+#include <fwbuilder/Interface.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -59,6 +60,9 @@ const char *FWObject::TYPENAME={"UNDEF"};
 string FWObject::NOT_FOUND="";
 
 
+//#define FWB_DEBUG
+//#define TI_DEBUG
+
 void FWObject::fromXML(xmlNodePtr root) throw(FWException)
 {
     assert(root!=NULL);
@@ -138,11 +142,10 @@ xmlNodePtr FWObject::toXML(xmlNodePtr parent, bool process_children)
 
     if (id!=-1)
     {
-        xmlAttrPtr pr = xmlNewProp(
+        xmlNewProp(
             me, 
             TOXMLCAST("id"),
             STRTOXMLCAST(s_id));
-        xmlAddID(NULL, parent->doc, STRTOXMLCAST(s_id), pr);
     }
 
     for(map<string, string>::const_iterator i=data.begin(); i!=data.end(); ++i) 
@@ -160,6 +163,7 @@ xmlNodePtr FWObject::toXML(xmlNodePtr parent, bool process_children)
         for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
             (*j)->toXML(me);
     }
+
     return me;
 }
 
@@ -179,6 +183,7 @@ FWObject::FWObject()
     setId(FWObjectDatabase::generateUniqueId());
 
     setDirty(false);
+    storeCreationTime();
 }
 
 FWObject::FWObject(bool new_id)
@@ -197,12 +202,14 @@ FWObject::FWObject(bool new_id)
         setId(FWObjectDatabase::generateUniqueId());
 
     setDirty(false);
+    storeCreationTime();
 }
 
 FWObject::FWObject(const FWObject &c) : list<FWObject*>(c)
 {
     init = false;
     *this = c;
+    storeCreationTime();
 }
 
 FWObject::FWObject(const FWObjectDatabase *root, bool )
@@ -220,13 +227,35 @@ FWObject::FWObject(const FWObjectDatabase *root, bool )
     setId(FWObjectDatabase::generateUniqueId());
 
     setDirty(false);
+    storeCreationTime();
 }
 
 FWObject::~FWObject() 
 {
     init = true;  // ignore read-only
-    deleteChildren();
+    destroyChildren();
     data.clear();
+    private_data.clear();
+    
+}
+
+void FWObject::setPrivateData(const string &key, void *data)
+{
+    private_data[key] = data;
+}
+
+void* FWObject::getPrivateData(const string &key) const
+{
+    map<string, void*>::const_iterator it = private_data.find(key);
+    if(it == private_data.end())
+        return NULL;
+    else
+        return it->second;
+}
+
+map<string, void*> &FWObject::getAllPrivateData()
+{
+    return private_data;
 }
 
 FWObject* FWObject::getParent() const
@@ -246,10 +275,23 @@ void FWObject::setXMLName(const string &n)
 
 FWObject* FWObject::_find(const string& name) const
 {
-    const_iterator i=find_if(begin(),end(), FWObjectNameEQPredicate(name));
+    const_iterator i = std::find_if(begin(),end(), FWObjectNameEQPredicate(name));
     return i==end()?NULL:(*i);
 }
 
+list<FWObject*> FWObject::findIf(FWObjectFindPredicate *pred)
+{
+    list<FWObject*> res_list;
+    list<FWObject*>::iterator i1;
+    for(i1=begin(); i1!=end(); ++i1)
+    {
+        if ((*pred)(*i1)) res_list.push_back(*i1);
+        list<FWObject*> res1 = (*i1)->findIf(pred);
+        res_list.splice(res_list.begin(), res1);
+    }
+    return res_list;
+}
+
 FWObject* FWObject::findObjectByName(const string &type,
                                      const string &name) throw(FWException)
 {
@@ -286,50 +328,51 @@ FWObject* FWObject::findObjectByAttribute(const std::string &attr,
 }
 
 
-bool FWObject::cmp(const FWObject *obj) throw(FWException)
+bool FWObject::cmp(const FWObject *obj, bool recursive) throw(FWException)
 {
+    if (getTypeName() != obj->getTypeName() || name != obj->name || comment != obj->comment || ro != obj->ro)
+        return false;
+
+    if (data.size() != obj->data.size())
+        return false;
+
     for(map<string, string>::const_iterator i=data.begin(); i!=data.end(); ++i) 
     {
         const string &name  = (*i).first;
         const string &value = (*i).second;
 // 10/21/2008 --vk
         map<string,string>::const_iterator j=obj->data.find(name);
-        if (j==obj->data.end()) continue;
+        if (j==obj->data.end()) return false;
         if (j->second!=value) return false;
     }
-    
-    if (size()!=obj->size())  return false;
+
+    if (recursive)
+    {
+        if (size()!=obj->size())  return false;
 
 /* chidren are not necessarily in the same order in two groups */
-    FWObject::const_iterator i1=begin();
-    for ( ; i1!=end(); ++i1)
-    {
-        bool found=false;
-//        FWObject *o2=NULL;
-        FWObject::const_iterator j1=obj->begin();
-        for ( ; j1!=obj->end(); ++j1)
+        std::set<FWObject*> matched;
+        FWObject::const_iterator i1=begin();
+        for ( ; i1!=end(); ++i1)
         {
-            if ((*i1)->cmp(*j1))
+            bool found_equal_child_object=false;
+            FWObject::const_iterator j1=obj->begin();
+            for ( ; j1!=obj->end(); ++j1)
             {
-                found=true;
-                break;
+                if ((*i1)->cmp(*j1, recursive))
+                {
+                    if (matched.find(*j1) == matched.end())
+                    {
+                        matched.insert(*j1);
+                        found_equal_child_object = true;
+                        break;
+                    }
+                    // else object matches some other object
+                }
             }
+            if (!found_equal_child_object) return false;
         }
-        if (!found) return false;
     }
-
-#if 0
-    list<FWObject*>::const_iterator i1=begin();
-    list<FWObject*>::const_iterator j1=obj->begin();
-    for( ; i1!=end() && j1!=obj->end(); ++i1,++j1)     
-    {
-        FWObject *o1=*i1;
-        FWObject *o2=*j1;
-        if ( ! o1->cmp(o2)) 
-            return false;
-    }
-    return ( i1==end() && j1==obj->end());
-#endif
     return true;
 }
 
@@ -403,6 +446,7 @@ FWObject& FWObject::shallowDuplicate(const FWObject *x, bool preserve_id)
     comment = x->comment;
     ro = x->ro;
     data = x->data;
+    private_data = x->private_data;
 
     setReadOnly(false);
 
@@ -427,6 +471,31 @@ FWObject& FWObject::shallowDuplicate(const FWObject *x, bool preserve_id)
     return *this;
 }
 
+class InheritsFWOptions: public std::unary_function<FWObject*, bool>
+{
+    public:
+    InheritsFWOptions() {}
+    bool operator()(const FWObject *o) const 
+    {
+        return FWOptions::constcast(o)!=NULL;
+    }
+};
+
+FWObject& FWObject::duplicateForUndo(const FWObject *obj) throw(FWException)
+{
+    setRO(false);
+    InheritsFWOptions pred;
+    FWObject::const_iterator mine_opts_iter = std::find_if(begin(), end(), pred);
+    FWObject::const_iterator their_opts_iter = std::find_if(obj->begin(), obj->end(), pred);
+    if (their_opts_iter != obj->end())
+    {
+        if (mine_opts_iter != end()) (*mine_opts_iter)->duplicate(*their_opts_iter);
+        else addCopyOf(*their_opts_iter);
+    }
+    shallowDuplicate(obj);
+    return *this;
+}
+
 const string &FWObject::getName() const 
 { 
     return name;
@@ -532,11 +601,15 @@ void FWObject::setStr(const string &name, const string &val)
 
 int FWObject::getInt(const string &name) const
 {
-    string s=getStr(name);
-    if(s!="") 
-        return( atol(s.c_str()) );
-    else   
-        return(-1);
+    string val = getStr(name);
+    size_t n = val.find_first_of(" \n\r\t");
+    while (n != string::npos)
+    {
+        val = val.erase(n, 1);
+        n = val.find_first_of(" \n\r\t");
+    }
+    if (val != "") return( atol(val.c_str()) );
+    else return(-1);
 }
 
 void FWObject::setInt(const string &name, int val)
@@ -554,6 +627,12 @@ bool FWObject::getBool(const string &name) const
 {
 // 10/21/2008 --vk
     string val = getStr(name);
+    size_t n = val.find_first_of(" \n\r\t");
+    while (n != string::npos)
+    {
+        val = val.erase(n, 1);
+        n = val.find_first_of(" \n\r\t");
+    }
     return (val=="1" || cxx_strcasecmp(val.c_str() , "true")==0);
 }
 
@@ -582,12 +661,12 @@ void FWObject::Hide()
 
 
 
-void FWObject::dump(bool recursive,bool brief,int offset) 
+void FWObject::dump(bool recursive,bool brief,int offset) const
 {
     dump(cerr,recursive,brief,offset);
 }
 
-void FWObject::dump(std::ostream &f,bool recursive,bool brief,int offset) 
+void FWObject::dump(std::ostream &f,bool recursive,bool brief,int offset) const
 {
     FWObject *o;
     string    n;
@@ -604,13 +683,10 @@ void FWObject::dump(std::ostream &f,bool recursive,bool brief,int offset)
             f << " Library=" << getLibrary();
 	f << " Root=" << getRoot();
         f << " ref_counter=" << ref_counter;
-
-	if (FWReference::constcast(this)!=0)
-	    f << " RefID=" << FWReference::cast(this)->getPointerId();
-
 	f << endl;
 
-	if (recursive) {
+	if (recursive)
+        {
 	    list<FWObject*>::const_iterator m;
 	    for (m=begin(); m!=end(); ++m) {
 		if (  (o=(*m))!=NULL)  o->dump(f,recursive,brief,offset+2);
@@ -762,39 +838,8 @@ void FWObject::swapObjects(FWObject *o1, FWObject *o2)
     setDirty(true);
 }
 
-/**
- * we move deleted objects to the special library with id
- * 'sysid99' rather than delete them completely. This serves two purposes:
- *
- * 1. can easily provide for undelete function which is very useful
- * 2. can catch a situation when an object has been deleted fromt he external
- *    library but is still used in the data file
- *
- */
-void FWObject::_moveToDeletedObjects(FWObject *obj)
-{
-    FWObjectDatabase *root = getRoot();
-    FWObject *dobj = root->findInIndex( FWObjectDatabase::DELETED_OBJECTS_ID );
-            
-    if (dobj==NULL)
-    {
-        dobj = root->createLibrary();
-        dobj->setId(FWObjectDatabase::DELETED_OBJECTS_ID);
-        dobj->setName("Deleted Objects");
-        dobj->setReadOnly(false);
-        root->add(dobj);
-    }
-
-/* just in case check if an object with the same id is already there */
-    if (dobj->getById( obj->getId(), false )==NULL &&
-        dobj->validateChild(obj) )   dobj->add(obj);
-}
-
 void FWObject::remove(FWObject *obj, bool delete_if_last)
 {
-//    if (getId()==FWObjectDatabase::cast(getRoot())->getDeletedObjectsId())
-//        return;
-
     FWObject::iterator fi=std::find(begin(), end(), obj);
     if(fi!=end())
     {
@@ -804,10 +849,12 @@ void FWObject::remove(FWObject *obj, bool delete_if_last)
         setDirty(true);
         obj->unref();
 
-        if (delete_if_last &&
-            obj->ref_counter==0 &&
-            getId()!=FWObjectDatabase::DELETED_OBJECTS_ID)
-            _moveToDeletedObjects(obj);
+        if (delete_if_last && obj->ref_counter==0)
+        {
+            FWObjectDatabase *db = getRoot();
+            if (db) db->removeFromIndex(obj->getId());
+            delete obj;
+        }
     }
 }
 
@@ -822,16 +869,44 @@ void FWObject::_removeAll(FWObject *rm)
 void FWObject::removeAllInstances(FWObject *rm)
 {
     checkReadOnly();
-
-    bool deletedObject =
-        (rm->getParent()->getId()==FWObjectDatabase::DELETED_OBJECTS_ID);
     removeAllReferences(rm);
     _removeAll(rm);
+}
 
-    if (!deletedObject) _moveToDeletedObjects(rm);
+void FWObject::removeRef(FWObject *obj)
+{
+    int  obj_id=obj->getId();
+    for(list<FWObject*>::iterator m=begin(); m!=end(); ++m) 
+    {
+        FWObject *o=*m;
+        FWReference *oref = FWReference::cast(o);
+        if (oref && oref->getPointerId()==obj_id)
+        {
+            // do not delete object even if this reference was the last one (?)
+            obj->unref();  
+
+            FWObject::remove(o, false);  // do not remove
+            delete o;
+            return;
+        }
+    }
+}
+
+void FWObject::_removeAllRef(FWObject *rm)
+{
+    // Do not delete references to the same object from its children
+    // such as references to the firewall or cluster in its own rules
+    if (this == rm || this->isChildOf(rm)) return;
+
+    for (FWObject::iterator i=begin(); i!=end(); i++)
+        (*i)->_removeAllRef(rm);
     
-//    if(!rm->ref_counter) 
-//        delete rm;
+    removeRef(rm);
+}
+
+void FWObject::removeAllReferences(FWObject *rm)
+{
+    _removeAllRef(rm);
 }
 
 void FWObject::findAllReferences(const FWObject *obj, std::set<FWReference*> &res)
@@ -859,38 +934,6 @@ set<FWReference*> FWObject::findAllReferences(const FWObject *obj)
     return res;
 }
 
-void FWObject::removeRef(FWObject *obj)
-{
-    int  obj_id=obj->getId();
-    for(list<FWObject*>::iterator m=begin(); m!=end(); ++m) 
-    {
-        FWObject *o=*m;
-        FWReference *oref = FWReference::cast(o);
-        if (oref && oref->getPointerId()==obj_id)
-        {
-            // do not delete object even if this reference was the last one (?)
-            obj->unref();  
-            // remove will delete o because reference counter goes to zero
-            FWObject::remove(o); 
-            return;
-        }
-    }
-}
-
-void FWObject::_removeAllRef(FWObject *rm)
-{
-    for (FWObject::iterator i=begin(); i!=end(); i++)
-        (*i)->_removeAllRef(rm);
-    
-    removeRef(rm);
-}
-
-void FWObject::removeAllReferences(FWObject *rm)
-{
-    _removeAllRef(rm);
-//    std::for_each(begin(), end(), RemoveReference(obj) );
-}
-
 bool FWObject::validateChild(FWObject *obj)
 { 
     return true;
@@ -909,17 +952,48 @@ bool FWObject::validateChild(FWObject *obj)
     return true;
 }
 
+/*
+ * this method deletes all children recursively regardless of their
+ * usage counter. We call this method form destructor.
+ */
 void FWObject::destroyChildren()
 {
     FWObjectDatabase *dbr = getRoot();
+    while (size() > 0)
+    {
+        FWObject *o = front();
+        if (o)
+        {
+            if (o->size()) o->destroyChildren();
+            if (dbr && !dbr->init) dbr->removeFromIndex( o->getId() );
+            delete o;
+        }
+        pop_front();
+    }
+    //clear();
+}
+
+/*
+ * Walks the tree, looking for objects that are referenced by two parents
+ */
+void FWObject::findDuplicateLinksInTree()
+{
     for(list<FWObject*>::iterator m=begin(); m!=end(); ++m) 
     {
-        FWObject *o=*m;
-        o->destroyChildren();
-        if (dbr) dbr->removeFromIndex( o->getId() );
-        delete o;
+        FWObject *o = *m;
+        if (o->getParent() != this)
+        {
+            cerr << "Object '" << o->getName() << "' (" << o->getTypeName() << ") "
+                 << " has two parents in the tree: "
+                 << o->getParent()->getRoot() << "::"
+                 << o->getParent()->getPath(true)
+                 << " and "
+                 << getRoot() << "::"
+                 << getPath(true)
+                 << endl;
+        }
+        o->findDuplicateLinksInTree();
     }
-    clear();
 }
 
 /*
@@ -945,44 +1019,43 @@ void FWObject::destroyChildren()
  */
 void FWObject::clearChildren(bool recursive)
 {
+#ifdef FWB_DEBUG
+    cerr << "FWObject::clearChildren" << endl;
+#endif
+
     FWObjectDatabase *dbr = getRoot();
 
     checkReadOnly();
 
+    int referenced_children = 0;
+    int total_children = 0;
+
     for(list<FWObject*>::iterator m=begin(); m!=end(); ++m) 
     {
-        FWObject *o=*m;
+        FWObject *o = *m;
+
+        total_children++;
+
         if (recursive) o->clearChildren(recursive);
         o->unref();
         if(o->ref_counter==0) 
         {
             if (dbr) dbr->removeFromIndex( o->getId() );
             delete o;
-        }
+        } else
+            referenced_children++;
     }
     clear();
     setDirty(true);
+
+#ifdef FWB_DEBUG
+    cerr << "Deleted " << total_children - referenced_children 
+         << " child objects; still referenced " << referenced_children
+         << " child objects" << endl;
+#endif
 }
 
-/*
- * this method deletes all children recursively regardless of their
- * usage counter. We call this method form destructor.
- */
-void FWObject::deleteChildren()
-{
-    FWObjectDatabase *dbr = getRoot();
-
-    for(list<FWObject*>::iterator m=begin(); m!=end(); ++m) 
-    {
-        FWObject *o=*m;
-        o->deleteChildren();
-        if (dbr && !dbr->init) dbr->removeFromIndex( o->getId() );
-        delete o;
-    }
-    clear();
-}
-
-int FWObject::getChildrenCount()
+int FWObject::getChildrenCount() const
 {
     return(size());
 }
@@ -994,7 +1067,7 @@ int FWObject::getChildrenCount()
 bool FWObject::isChildOf(FWObject *obj)
 {
     if (this==obj) return false;
-#if 0
+#ifdef FWB_DEBUG
     cerr << "FWObject::isChildOf" << endl;
     cerr << "this: " << endl;
     dump(true,true);
@@ -1008,6 +1081,16 @@ bool FWObject::isChildOf(FWObject *obj)
     return (p==obj);
 }
 
+bool FWObject::hasChild(FWObject *obj)
+{
+    int o_id = obj->getId();
+    for (list<FWObject*>::iterator it=begin(); it!=end(); ++it)
+    {
+        if ((*it)->getId() == o_id) return true;
+    }
+    return false;
+}
+
 FWObject* FWObject::getById  (int id, bool recursive)
 {
     if(id==getId())  return this;
@@ -1194,17 +1277,19 @@ FWObject::tree_iterator FWObject::tree_iterator::operator++(int )
 /* this is a prefix operator */
 FWObject::tree_iterator& FWObject::tree_iterator::operator++()
 {
-    if (node==(FWObject*)(-1)) return *this;
+    if (node == (FWObject*)(-1)) return *this;
 
 #ifdef TI_DEBUG
+    cerr << endl;
     cerr << "ENTRY  node=" << node << "("
          << node->getTypeName() << " " << node->getName() << ")"
          << endl;
+    node->dump(false, false);
 #endif
 
     if (node->size()!=0)
     {
-        node= node->front();
+        node = node->front();
 
 #ifdef TI_DEBUG
         cerr << "#2     node=" << node << "("
@@ -1213,10 +1298,10 @@ FWObject::tree_iterator& FWObject::tree_iterator::operator++()
         return *this;
     }
 
-    FWObject *p=node;
+    FWObject *p = node;
     while (node->getParent()!=NULL)
     {
-        p=node->getParent();
+        p = node->getParent();
 
 #ifdef TI_DEBUG
         cerr << "       p=" << p << "("
@@ -1224,7 +1309,13 @@ FWObject::tree_iterator& FWObject::tree_iterator::operator++()
 #endif
 
         for (FWObject::iterator i=p->begin(); i!=p->end(); ++i)
-            if ( node==(*i) )
+        {
+#ifdef TI_DEBUG
+            cerr << "       child=" << (*i) << "("
+                 << (*i)->getTypeName() << " " << (*i)->getName() << ")" << endl;
+#endif
+
+            if ( node == (*i) )
             {
                 ++i;
                 if (i==p->end())
@@ -1232,7 +1323,7 @@ FWObject::tree_iterator& FWObject::tree_iterator::operator++()
 #ifdef TI_DEBUG
                     cerr << "      end of list" << endl;
 #endif
-                    node=p;
+                    node = p;
                     break;
                 }
 
@@ -1245,6 +1336,7 @@ FWObject::tree_iterator& FWObject::tree_iterator::operator++()
 #endif
                 return *this;
             }
+        }
     }
     node=(FWObject*)(-1);
 
@@ -1285,18 +1377,19 @@ FWObject::tree_iterator FWObject::tree_end()
 int FWObject::replaceRef(int old_id, int new_id)
 {
     int ref_replacement_counter = 0;
-    _replaceRef(this, old_id, new_id, ref_replacement_counter);
+    replaceReferenceInternal(old_id, new_id, ref_replacement_counter);
     return ref_replacement_counter;
 }
 
-void FWObject::_replaceRef(FWObject *rs, int old_id, int new_id,
-                           int &counter)
+void FWObject::replaceReferenceInternal(int old_id, int new_id, int &counter)
 {
-    FWReference *ref = FWReference::cast(rs);
+    if (old_id == new_id) return;
+
+    FWReference *ref = FWReference::cast(this);
     if (ref==NULL)
     {
-        for (FWObject::iterator j1=rs->begin(); j1!=rs->end(); ++j1)
-            _replaceRef( *j1, old_id, new_id, counter);
+        for (FWObject::iterator j1=begin(); j1!=end(); ++j1)
+            (*j1)->replaceReferenceInternal(old_id, new_id, counter);
     } else
     { 
         if (ref->getPointerId()==old_id)
