@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWOptions.cpp 245 2009-01-18 22:24:54Z vadim $
+  $Id: FWOptions.cpp 548 2010-02-15 23:56:04Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -68,9 +68,9 @@ void FWOptions::fromXML(xmlNodePtr root) throw(FWException)
             if (cont)
             {
                 setStr(n, cont );
-                FREEXMLBUFF(n);
                 FREEXMLBUFF(cont);
             }
+            FREEXMLBUFF(n);
         }
     }
 }
@@ -115,6 +115,17 @@ FirewallOptions::FirewallOptions(const FWObjectDatabase *root,
                                  bool prepopulate) :
     FWOptions(root,prepopulate) {}
 
+const char *InterfaceOptions::TYPENAME  ={"InterfaceOptions"};
+InterfaceOptions::InterfaceOptions() : FWOptions() {}
+InterfaceOptions::InterfaceOptions(const FWObjectDatabase *root,bool prepopulate) :
+    FWOptions(root, prepopulate) {}
+
+const char *RuleSetOptions::TYPENAME={"RuleSetOptions"};
+RuleSetOptions::RuleSetOptions() : FWOptions() {}
+RuleSetOptions::RuleSetOptions(const FWObjectDatabase *root,
+                               bool prepopulate) :
+    FWOptions(root,prepopulate) {}
+
 const char *PolicyRuleOptions::TYPENAME={"PolicyRuleOptions"};
 PolicyRuleOptions::PolicyRuleOptions() : FWOptions() {}
 PolicyRuleOptions::PolicyRuleOptions(const FWObjectDatabase *root,
@@ -122,7 +133,6 @@ PolicyRuleOptions::PolicyRuleOptions(const FWObjectDatabase *root,
     FWOptions(root,prepopulate) {}
 
 const char *NATRuleOptions::TYPENAME={"NATRuleOptions"};
-
 NATRuleOptions::NATRuleOptions() : FWOptions() {}
 NATRuleOptions::NATRuleOptions(const FWObjectDatabase *root, bool prepopulate) :
     FWOptions(root,prepopulate) {}
@@ -133,3 +143,10 @@ RoutingRuleOptions::RoutingRuleOptions(const FWObjectDatabase *root,
                                        bool prepopulate) :
     FWOptions(root,prepopulate) {}
 
+const char *ClusterGroupOptions::TYPENAME={"ClusterGroupOptions"};
+ClusterGroupOptions::ClusterGroupOptions() : FWOptions() {}
+ClusterGroupOptions::ClusterGroupOptions(const FWObjectDatabase *root,
+                                         bool prepopulate) :
+    FWOptions(root,prepopulate) {}
+
+
