@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWReference.cpp 353 2009-06-08 21:33:04Z vadim $
+  $Id: FWReference.cpp 613 2010-06-13 03:42:22Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -87,13 +87,10 @@ xmlNodePtr FWReference::toXML(xmlNodePtr parent) throw(FWException)
     if (int_ref == -1 && !str_ref.empty())
         int_ref = FWObjectDatabase::getIntId(str_ref);
 
-    if (int_ref > -1 && str_ref.empty())
-        str_ref = FWObjectDatabase::getStringId(int_ref);
+    str_ref = FWObjectDatabase::getStringId(int_ref);
 
-    xmlAttrPtr pr = xmlNewProp(me, 
-                               TOXMLCAST("ref"),
-                               STRTOXMLCAST(str_ref));
-    xmlAddRef(NULL, parent->doc, STRTOXMLCAST(str_ref), pr);
+    xmlNewProp(me, TOXMLCAST("ref"), STRTOXMLCAST(str_ref));
+    //xmlAddRef(NULL, parent->doc, STRTOXMLCAST(str_ref), pr);
 
     return me;
 }
@@ -107,6 +104,14 @@ FWObject& FWReference::shallowDuplicate(const FWObject *_other,
     return *this;
 }
 
+bool FWReference::cmp(const FWObject *obj, bool /* UNUSED recursive */) throw(FWException)
+{
+    const FWReference *rx = FWReference::constcast(obj);
+    if (rx == NULL) return false;
+    if (int_ref != rx->int_ref || str_ref != rx->str_ref) return false;
+    return true;
+}
+
 void FWReference::add(FWObject*)
 {
     throw std::string("Can't add to a reference !");
@@ -149,13 +154,13 @@ int FWReference::getPointerId()
 }
 
 
-void FWReference::dump(std::ostream &f, bool recursive, bool brief, int offset)
+void FWReference::dump(std::ostream &f, bool recursive, bool brief, int offset) const
 {
     FWObject::dump(f, recursive, brief, offset);
 
-    FWObject* ptr=getPointer();
+    FWObject* ptr = getRoot()->findInIndex(int_ref);
 
-    f << string(offset,' ') << "PointerId: " << getPointerId() << endl;
+    f << string(offset,' ') << "PointerId: " << int_ref << endl;
     f << string(offset,' ') << "Pointer: " << ptr << endl;
     if (ptr)
     {
