@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Host.cpp 236 2008-12-25 20:36:41Z vadim $
+  $Id: Host.cpp 432 2009-09-23 16:22:20Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -133,10 +133,11 @@ Management *Host::getManagementObject()
  */
 const InetAddr* Host::getManagementAddress() throw(FWException)
 {
-    FWObjectTypedChildIterator j = findByType(Interface::TYPENAME);
-    for( ; j!=j.end(); ++j)
+    list<FWObject*> interfaces = getByTypeDeep(Interface::TYPENAME);
+    list<FWObject*>::iterator i;
+    for (i=interfaces.begin(); i!=interfaces.end(); ++i )
     {
-        Interface *iface = Interface::cast(*j);
+        Interface *iface = Interface::cast(*i);
         if (iface->isManagement())
         {
             FWObjectTypedChildIterator k = iface->findByType(IPv4::TYPENAME);
