@@ -4,9 +4,9 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Host.h 206 2008-10-18 20:36:18Z vadim $
+  $Id: Host.h 534 2010-02-07 23:25:06Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -33,6 +33,7 @@
 #include <fwbuilder/Address.h>
 #include <fwbuilder/IPv4.h>
 #include <fwbuilder/FWOptions.h>
+#include <fwbuilder/ObjectMatcher.h>
 
 namespace libfwbuilder
 {
@@ -47,7 +48,7 @@ class Host : public Address
     public:
 
     Host();
-    Host(const FWObject *root,bool prepopulate);
+    Host(const FWObjectDatabase *root,bool prepopulate);
     virtual ~Host();
 
     virtual void fromXML(xmlNodePtr parent) throw(FWException);
@@ -56,6 +57,8 @@ class Host : public Address
 
     DECLARE_FWOBJECT_SUBTYPE(Host);
 
+    DECLARE_DISPATCH_METHODS(Host);
+    
     /*
      * verify whether given object type is approppriate as a child
      */
@@ -86,7 +89,9 @@ class Host : public Address
      * similar to hasInetAddress() but counts addresses
      */
     virtual int countInetAddresses(bool skip_loopback) const;
-    
+
+    virtual bool isPrimaryObject() const { return true; }
+
     protected:
     
     Management *mgmt;
