@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ICMP6Service.cpp 236 2008-12-25 20:36:41Z vadim $
+  $Id: ICMP6Service.cpp 590 2010-03-26 03:06:41Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -47,6 +47,6 @@ ICMP6Service::ICMP6Service(const FWObjectDatabase *root, bool prepopulate) :
 
 ICMP6Service::~ICMP6Service() {}
 
-string ICMP6Service::getProtocolName()    {    return "icmp6";}
-int    ICMP6Service::getProtocolNumber()  {    return 58;     }
+string ICMP6Service::getProtocolName()          {    return "icmp6";}
+int    ICMP6Service::getProtocolNumber() const  {    return 58;     }
 
