@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ICMPService.cpp 236 2008-12-25 20:36:41Z vadim $
+  $Id: ICMPService.cpp 590 2010-03-26 03:06:41Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -50,8 +50,8 @@ ICMPService::ICMPService(const FWObjectDatabase *root,bool prepopulate) : Servic
 
 ICMPService::~ICMPService() {}
 
-string ICMPService::getProtocolName()    {    return "icmp";}
-int    ICMPService::getProtocolNumber()  {    return 1;     }
+string ICMPService::getProtocolName()          {    return "icmp";}
+int    ICMPService::getProtocolNumber() const  {    return 1;     }
 
 void ICMPService::fromXML(xmlNodePtr root) throw(FWException)
 {
