@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: IPv4.cpp 236 2008-12-25 20:36:41Z vadim $
+  $Id: IPv4.cpp 468 2009-11-18 20:05:31Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -141,7 +141,7 @@ void IPv4::setAddressNetmask(const std::string& s)
     inet_addr_mask = new InetAddrMask(s);
 }
 
-void IPv4::dump(std::ostream &f,bool recursive,bool brief,int offset)
+void IPv4::dump(std::ostream &f,bool recursive,bool brief,int offset) const
 {
     FWObject::dump(f, recursive, brief, offset);
     f << inet_addr_mask->getAddressPtr()->toString() << endl;
