@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Interval.cpp 236 2008-12-25 20:36:41Z vadim $
+  $Id: Interval.cpp 561 2010-02-24 00:21:07Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -318,7 +318,7 @@ FWReference* Interval::createRef()
     return ref;
 }
 
-bool Interval::isAny()
+bool Interval::isAny() const
 { 
     return getId()==FWObjectDatabase::ANY_INTERVAL_ID;
 }
