@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland <vadim@fwbuilder.org>
 
-  $Id: MultiAddress.cpp 236 2008-12-25 20:36:41Z vadim $
+  $Id: MultiAddress.cpp 502 2010-01-09 22:59:52Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -92,7 +92,7 @@ bool MultiAddress::validateChild(FWObject *o)
     return ObjectGroup::validateChild(o);
 }
 
-void MultiAddress::loadFromSource(bool) throw(FWException)
+void MultiAddress::loadFromSource(bool, bool) throw(FWException)
 {
     cerr << "virtual function MultiAddress::loadFromSource is not implemented"
          << endl;
