@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Resources.cpp 234 2008-12-23 06:17:53Z vadim $
+  $Id: Resources.cpp 548 2010-02-15 23:56:04Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -49,6 +49,7 @@
 #include "fwbuilder/RuleElement.h"
 #include "fwbuilder/FWOptions.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/Interface.h"
 #include "fwbuilder/Host.h"
 #include "fwbuilder/Tools.h"
 
@@ -70,18 +71,40 @@ Resources::Resources() throw(FWException)
 
 Resources::Resources(const string &_resF) throw(FWException)
 {
-    doc=NULL;
-    resfile=_resF;
+    doc = NULL;
+    resfile = _resF;
 
     if (global_res==NULL) 
     {
-        global_res=this;
+        global_res = this;
         loadRes(_resF);
         loadSystemResources();
     } else
         loadRes(_resF);
 }
 
+Resources::~Resources()
+{
+    if (doc) xmlFreeDoc(doc);
+}
+
+void Resources::clear()
+{
+    for (map<string,Resources*>::iterator it=platform_res.begin(); 
+         it!=platform_res.end(); ++it)
+    {
+        delete it->second;
+    }
+    platform_res.clear();
+
+    for (map<string,Resources*>::iterator it=os_res.begin(); 
+         it!=os_res.end(); ++it)
+    {
+        delete it->second;
+    }
+    os_res.clear();
+}
+
 string Resources::getXmlNodeContent(xmlNodePtr node)
 {
     string res;
@@ -115,9 +138,9 @@ void Resources::loadRes(const std::string &rfile ) throw(FWException)
 //    doc = xmlParseFile(rfile.c_str()); 
 //    doc = XMLTools::loadAndParseFile(rfile);
 
-    if(!doc)   throw FWException("Error parsing "+rfile);
+    if (!doc) throw FWException("Error parsing "+rfile);
 
-    root=xmlDocGetRootElement(doc);
+    root = xmlDocGetRootElement(doc);
    
     if(!root || !root->name || strcmp(FROMXMLCAST(root->name), "FWBuilderResources")!=0) 
     {
@@ -139,27 +162,27 @@ void Resources::loadSystemResources() throw(FWException)
     string::size_type n=resfile.find_last_of("/\\");
     string resDir = resfile.substr(0,n);
 
-	list<string> pllist = getDirList( resDir + FS_SEPARATOR +PLATFORM_RES_DIR_NAME,
-									"xml" );
+    list<string> pllist = getDirList( resDir + FS_SEPARATOR +PLATFORM_RES_DIR_NAME,
+                                      "xml" );
 
-	for (list<string>::iterator lsi1=pllist.begin(); lsi1!=pllist.end(); lsi1++)
-	{
-		string::size_type n=lsi1->find_last_of("/\\")+1;
-		string platform=lsi1->substr(n, lsi1->rfind(".xml")-n);
-	    Resources *tr=new Resources(*lsi1);	
+    for (list<string>::iterator lsi1=pllist.begin(); lsi1!=pllist.end(); lsi1++)
+    {
+        string::size_type n=lsi1->find_last_of("/\\")+1;
+        string platform=lsi1->substr(n, lsi1->rfind(".xml")-n);
+        Resources *tr=new Resources(*lsi1);	
         platform_res[platform]=tr;
-	}
+    }
 
 
-	list<string> oslist = getDirList( resDir + FS_SEPARATOR +OS_RES_DIR_NAME,
-									"xml" );
-	for (list<string>::iterator lsi2=oslist.begin(); lsi2!=oslist.end(); lsi2++)
-	{
-		string::size_type n=lsi2->find_last_of("/\\")+1;
-		string os=lsi2->substr(n, lsi2->rfind(".xml")-n);
-	    Resources *tr=new Resources(*lsi2);	
+    list<string> oslist = getDirList( resDir + FS_SEPARATOR +OS_RES_DIR_NAME,
+                                      "xml" );
+    for (list<string>::iterator lsi2=oslist.begin(); lsi2!=oslist.end(); lsi2++)
+    {
+        string::size_type n=lsi2->find_last_of("/\\")+1;
+        string os=lsi2->substr(n, lsi2->rfind(".xml")-n);
+        Resources *tr=new Resources(*lsi2);	
         os_res[os]=tr;
-	}
+    }
 
 #if 0
     cerr << "Loaded resources for the following modules :\n";
@@ -219,6 +242,36 @@ bool    Resources::getResourceBool(const string& resource_path)
     return (res=="true" || res=="True");
 }
 
+/*
+ * Adds bodies of xml elements found directly under resource_path to
+ * the list<string> res
+ *
+ * <element1>
+ *   <element2>
+ *      <string>value1</string>
+ *      <string>value2</string>
+ *      <string>value3</string>
+ *      <string>value4</string>
+ *   </element2>
+ * </element1>
+ *
+ * here resorce_path="/element1/element2", returned list consists of
+ * strings value1,value2,value3,value4
+ */
+void Resources::getResourceStrList(const string& resource_path, list<string> &res)
+{
+    xmlNodePtr node = XMLTools::getXmlNodeByPath(root, resource_path.c_str());
+    if (node)
+    {
+        xmlNodePtr c;
+        for(c=node->xmlChildrenNode; c; c=c->next) 
+        {
+            if ( xmlIsBlankNode(c) ) continue;
+            res.push_back(getXmlNodeContent(c));
+        }
+    }
+}
+
 string  Resources::getObjResourceStr(const FWObject *obj,
                                      const string& resource_name)
 {
@@ -280,37 +333,54 @@ string  Resources::getInstaller()
     return getResourceStr("/FWBuilderResources/Target/installer");
 }
 
-vector<string>    Resources::getListOfPlatforms()
+string  Resources::getTransferAgent()
+{
+    return getResourceStr("/FWBuilderResources/Target/transfer_agent");
+}
+
+vector<string> Resources::getListOfPlatforms()
 {
     vector<string> vs;
-    for (map<string,Resources*>::iterator i1=platform_res.begin();
-         i1!=platform_res.end(); ++i1)
-        vs.push_back( (*i1).first );
-    return vs;
-}
-
-
-
-map<string,string>    Resources::getPlatforms()
-{
-    map<string,string> vs;
     for (map<string,Resources*>::iterator i1=platform_res.begin();
          i1!=platform_res.end(); ++i1)
     {
-        string desc=(*i1).second->getResourceStr("/FWBuilderResources/Target/description");
-        vs[ (*i1).first ]=desc;
+        vs.push_back( (*i1).first );
     }
     return vs;
 }
 
 
-map<string,string>    Resources::getOS()
+
+map<string,string> Resources::getPlatforms()
+{
+    map<string,string> vs;
+    for (map<string,Resources*>::iterator i1=platform_res.begin();
+         i1!=platform_res.end(); ++i1)
+    {
+        Resources *res = (*i1).second;
+        if (res)
+        {
+            string desc = res->getResourceStr(
+                "/FWBuilderResources/Target/description");
+            vs[ (*i1).first ]=desc;
+        }
+    }
+    return vs;
+}
+
+
+map<string,string> Resources::getOS()
 {
     map<string,string> vs;
     for (map<string,Resources*>::iterator i1=os_res.begin(); i1!=os_res.end(); ++i1)
     {
-        string desc=(*i1).second->getResourceStr("/FWBuilderResources/Target/description");
-        vs[ (*i1).first ]=desc;
+        Resources *res = (*i1).second;
+        if (res)
+        {
+            string desc = res->getResourceStr(
+                "/FWBuilderResources/Target/description");
+            vs[ (*i1).first ]=desc;
+        }
     }
     return vs;
 }
@@ -428,6 +498,26 @@ void    Resources::setDefaultTargetOptions(const string &target,Firewall *fw)  t
     r->setDefaultOptionsAll(opt,"/FWBuilderResources/Target/options/default");
 }
 
+void    Resources::setDefaultIfaceOptions(const string &target,Interface *iface)
+        throw (FWException)
+{
+    FWOptions *opt=iface->getOptionsObject();
+    /* if InterfaceOptions object does not yet exist -> create one */
+    if (opt == NULL) {
+        iface->add(iface->getRoot()->create(InterfaceOptions::TYPENAME));
+        opt = iface->getOptionsObject();
+    }
+
+    Resources *r=NULL;
+
+    if (platform_res.count(target)!=0)      r=platform_res[target];
+    if (r==NULL && os_res.count(target)!=0) r=os_res[target];
+    if (r==NULL)
+        throw FWException("Support module for target '"+target+"' is not available");
+
+    r->setDefaultOptionsAll(opt,"/FWBuilderResources/Target/options/interface");
+}
+
 void    Resources::setDefaultOptions(Host *h)
 {
     FWOptions *opt=h->getOptionsObject();
@@ -462,33 +552,23 @@ bool Resources::getTargetCapabilityBool(const string &target,
     return (s=="true" || s=="True");
 }
 
-bool Resources::isTargetActionSupported(const string &target,
-                                        const string &action)
+bool Resources::isTargetActionSupported(const string &target, const string &action)
 {
     bool res=false;
     try  
     {
-         res=getTargetCapabilityBool(target,
-                 "actions/"+action+"/supported");
-
-    } catch (FWException &ex)
-    {
-    }
+         res = getTargetCapabilityBool(target, "actions/" + action + "/supported");
+    } catch (FWException &ex) { }
     return res;
 }
 
-string Resources::getActionEditor(const string &target,
-                                        const string &action)
+string Resources::getActionEditor(const string &target, const string &action)
 {
     string res="None";
     try  
     {
-         res=getTargetCapabilityStr(
-                 target,"actions/"+action+"/parameter");
-
-    } catch (FWException &ex)
-    {
-    }
+         res = getTargetCapabilityStr(target, "actions/" + action + "/parameter");
+    } catch (FWException &ex) { }
     return res;
 }
 
