@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Rule.cpp 370 2009-06-24 16:00:58Z vadim $
+  $Id: Rule.cpp 475 2009-11-25 02:53:58Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -97,6 +97,16 @@ FWObject& Rule::shallowDuplicate(const FWObject *x,
     return  FWObject::shallowDuplicate(x,preserve_id);
 }
 
+bool Rule::cmp(const FWObject *x, bool recursive) throw(FWException)
+{
+    const Rule *rx = Rule::constcast(x);
+    if (fallback != rx->fallback ||
+        hidden != rx->hidden ||
+        label != rx->label ||
+        unique_id != rx->unique_id) return false;
+
+    return  FWObject::cmp(x, recursive);
+}
 
 /***************************************************************************/
 
@@ -168,6 +178,16 @@ FWObject& PolicyRule::shallowDuplicate(const FWObject *x,
     return  Rule::shallowDuplicate(x, preserve_id);
 }
 
+bool PolicyRule::cmp(const FWObject *x, bool recursive) throw(FWException)
+{
+    const PolicyRule *rx = PolicyRule::constcast(x);
+    if (rx == NULL) return false;
+    if (getDirection() != rx->getDirection() ||
+        getAction() != rx->getAction() ||
+        getLogging() != rx->getLogging()) return false;
+    return  Rule::cmp(x, recursive);
+}
+
 // <!ELEMENT PolicyRule (Src,Dst,Srv?,Itf?,When?,PolicyRuleOptions?)>
 RuleElementSrc*  PolicyRule::getSrc()
 {
@@ -229,7 +249,13 @@ bool PolicyRule::isEmpty()
 
 string PolicyRule::getActionAsString() const
 {
-    switch (action) {
+    return getActionAsString(action);
+}
+
+// static method
+string PolicyRule::getActionAsString(int act)
+{
+    switch (act) {
     case Accept:     return "Accept";
     case Deny:       return "Deny";
     case Reject:     return "Reject";
@@ -276,7 +302,7 @@ string PolicyRule::getDirectionAsString() const
     {
     case Inbound:   return "Inbound";
     case Outbound:  return "Outbound";
-    case Both:      return "Both";
+    default:        return "Both";
     }
     return "Both";
 }
@@ -396,10 +422,11 @@ FWOptions* PolicyRule::getOptionsObject()
     return FWOptions::cast( getFirstByType(PolicyRuleOptions::TYPENAME) );
 }
 
-RuleSet*   PolicyRule::getBranch()
+RuleSet* PolicyRule::getBranch()
 {
+    if (getAction() != PolicyRule::Branch) return NULL;
     FWObject *fw = this;
-    while (fw && !Firewall::isA(fw)) fw = fw->getParent();
+    while (fw && Firewall::cast(fw) == NULL) fw = fw->getParent();
     assert(fw!=NULL);
     string branch_id = getOptionsObject()->getStr("branch_id");
     if (!branch_id.empty())
@@ -486,13 +513,33 @@ void PolicyRule::removeRef(FWObject *obj)
     FWObject::removeRef(obj);
 }
 
+void PolicyRule::replaceReferenceInternal(int old_id, int new_id, int &counter)
+{
+    if (old_id == new_id) return;
+
+    FWObject::replaceReferenceInternal(old_id, new_id, counter);
+
+    string branch_id = getOptionsObject()->getStr("branch_id");
+    if (!branch_id.empty())
+    {
+        int branch_id_int = FWObjectDatabase::getIntId(branch_id);
+        if (branch_id_int == old_id)
+        {
+            getOptionsObject()->setStr("branch_id",
+                                       FWObjectDatabase::getStringId(new_id));
+            counter++;
+        }
+    }
+}
+
 /***************************************************************************/
 
 const char *NATRule::TYPENAME={"NATRule"};
 
 NATRule::NATRule() : Rule()
 {
-    rule_type=Unknown;
+    rule_type = Unknown;
+    setAction(NATRule::Translate);
 
     osrc_re = NULL;
     odst_re = NULL;
@@ -505,7 +552,8 @@ NATRule::NATRule() : Rule()
 
 NATRule::NATRule(const FWObjectDatabase *root,bool prepopulate) : Rule(root,prepopulate)
 {
-    rule_type=Unknown;
+    rule_type = Unknown;
+    setAction(NATRule::Translate);
 
     osrc_re = NULL;
     odst_re = NULL;
@@ -582,6 +630,26 @@ RuleElementInterval* NATRule::getWhen()
     return when_re;
 }
 
+string NATRule::getActionAsString() const
+{
+    return getActionAsString(action);
+}
+
+// static method
+string NATRule::getActionAsString(int act)
+{
+    if (act == Branch) return "NATBranch";
+    return "Translate";
+}
+
+void NATRule::setAction(const string& act)
+{
+    if (act=="Translate")  { setAction(Translate); return; }
+    if (act=="Branch")     { setAction(Branch); return; }
+    if (act=="NATBranch")  { setAction(Branch); return; }
+    setAction(Translate);
+}
+
 bool NATRule::isEmpty()
 {
     RuleElement *osrc=getOSrc();
@@ -601,6 +669,13 @@ void NATRule::fromXML(xmlNodePtr root) throw(FWException)
 
     FWObject::fromXML(root);
 
+    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("action")));
+    if(n)
+    {
+        setAction(string(n));
+        FREEXMLBUFF(n);
+    }
+
     n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("disabled")));
     if(n)
     {
@@ -628,6 +703,7 @@ xmlNodePtr NATRule::toXML(xmlNodePtr parent) throw(FWException)
 {
     xmlNodePtr me = FWObject::toXML(parent, false);
 //    xmlNewProp(me, TOXMLCAST("name"), STRTOXMLCAST(getName()));
+    xmlNewProp(me, TOXMLCAST("action"), STRTOXMLCAST(getActionAsString()));
     xmlNewProp(me, TOXMLCAST("comment"), STRTOXMLCAST(getComment()));
 
     FWObject *o;
@@ -667,8 +743,9 @@ FWOptions* NATRule::getOptionsObject()
     return FWOptions::cast( getFirstByType(NATRuleOptions::TYPENAME) );
 }
 
-RuleSet*   NATRule::getBranch()
+RuleSet* NATRule::getBranch()
 {
+    if (getAction() != NATRule::Branch) return NULL;
     FWObject *fw = getParent()->getParent();
     assert(fw!=NULL);
     string branch_id = getOptionsObject()->getStr("branch_id");
@@ -699,7 +776,7 @@ NATRule::NATRuleTypes NATRule::getRuleType() const
     return rule_type;
 }
 
-string  NATRule::getRuleTypeAsString() const 
+string NATRule::getRuleTypeAsString() const 
 {
     switch (rule_type) {
     case SNAT:     return("SNAT");     
@@ -714,11 +791,12 @@ string  NATRule::getRuleTypeAsString() const
     case Continue: return("Continue"); 
     case LB:       return("LB");       
     case NONAT:    return("NONAT");    
+    case NATBranch: return("NATBranch");    
     default:       return("Unknown");  
     }
 }
 
-void         NATRule::setRuleType(NATRuleTypes rt) 
+void NATRule::setRuleType(NATRuleTypes rt) 
 { 
     rule_type=rt;
 }
@@ -726,8 +804,9 @@ void         NATRule::setRuleType(NATRuleTypes rt)
 FWObject& NATRule::shallowDuplicate(const FWObject *x,
                                     bool preserve_id) throw(FWException)
 {
-    const NATRule *rx=NATRule::constcast(x);
-    if (rx!=NULL) rule_type=rx->rule_type;
+    const NATRule *rx = NATRule::constcast(x);
+    if (rx!=NULL) rule_type = rx->rule_type;
+    setAction(rx->getAction());
 
     osrc_re = NULL;
     odst_re = NULL;
@@ -740,7 +819,13 @@ FWObject& NATRule::shallowDuplicate(const FWObject *x,
     return  Rule::shallowDuplicate(x, preserve_id);
 }
 
-
+bool NATRule::cmp(const FWObject *x, bool recursive) throw(FWException)
+{
+    const NATRule *rx = NATRule::constcast(x);
+    if (rx == NULL) return false;
+    if (getAction() != rx->getAction()) return false;
+    return  Rule::cmp(x, recursive);
+}
 
 /***************************************************************************/
 
