@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: RuleElement.cpp 268 2009-04-21 03:05:51Z vadim $
+  $Id: RuleElement.cpp 593 2010-03-29 19:07:48Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -62,7 +62,7 @@ RuleElement::RuleElement(const FWObjectDatabase *root, bool prepopulate) :
     FWObject(root,prepopulate)
 {
     setNeg(false);
-    setId(-1);
+//    setId(-1);
 }
 
 void RuleElement::fromXML(xmlNodePtr root) throw(FWException)
@@ -81,9 +81,14 @@ void RuleElement::fromXML(xmlNodePtr root) throw(FWException)
 
 xmlNodePtr RuleElement::toXML(xmlNodePtr xml_parent_node) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(xml_parent_node);
+    int my_id = getId();
+    setId(-1);
+    // FWObject::toXML() skips id if it is == -1. RuleElement objects
+    // have no ID as per DTD (why?)
+    xmlNodePtr me = FWObject::toXML(xml_parent_node, false);
     xmlNewProp(me, TOXMLCAST("neg"),
                TOXMLCAST(((getNeg()) ? "True" : "False")));
+    setId(my_id);
     return me;
 }
 
@@ -118,18 +123,11 @@ void RuleElement::removeRef(FWObject *obj)
     }
 }
 
-bool RuleElement::isAny()
+bool RuleElement::isAny() const
 {
     if (getChildrenCount()!=1) return(false);
-
-    FWObject *o;
-    list<FWObject*>::iterator m = begin();
-    if (  (o=(*m))!=NULL ) 
-    {
-        if ((FWReference::cast(o))->getPointerId() == getAnyElementId())
-            return(true);
-    }
-    return(false);
+    const FWObject *o = front();
+    return ((FWReference::constcast(o))->getPointerIdDirect() == getAnyElementId());
 }
 
 
@@ -172,7 +170,7 @@ void RuleElement::_initialize(const FWObjectDatabase *root)
         FWObject::addRef( any_obj );
 }
 
-int RuleElement::getAnyElementId()
+int RuleElement::getAnyElementId() const
 {
     return -1;
 }
@@ -186,15 +184,14 @@ RuleElementSrc::RuleElementSrc(const FWObjectDatabase *root,bool prepopulate) :
         _initialize(root);
 }
 
-int RuleElementSrc::getAnyElementId() {
+int RuleElementSrc::getAnyElementId() const
+{
     return FWObjectDatabase::ANY_ADDRESS_ID;
 }
 
 xmlNodePtr RuleElementSrc::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -220,15 +217,14 @@ RuleElementDst::RuleElementDst(const FWObjectDatabase *root,bool prepopulate) :
         _initialize(root);
 }
 
-int RuleElementDst::getAnyElementId() {
+int RuleElementDst::getAnyElementId() const 
+{
     return FWObjectDatabase::ANY_ADDRESS_ID;
 }
 
 xmlNodePtr RuleElementDst::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -255,15 +251,14 @@ RuleElementSrv::RuleElementSrv(const FWObjectDatabase *root,bool prepopulate) :
 }
 
 
-int RuleElementSrv::getAnyElementId() {
+int RuleElementSrv::getAnyElementId() const
+{
     return FWObjectDatabase::ANY_SERVICE_ID;
 }
 
 xmlNodePtr RuleElementSrv::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -288,16 +283,14 @@ RuleElementItf::RuleElementItf(const FWObjectDatabase *root,bool prepopulate) :
         _initialize(root);
 }
 
-int RuleElementItf::getAnyElementId()
+int RuleElementItf::getAnyElementId() const
 {
     return FWObjectDatabase::ANY_ADDRESS_ID;
 }
 
 xmlNodePtr RuleElementItf::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -310,8 +303,9 @@ bool RuleElementItf::validateChild(FWObject *o)
 {
     if (FWObjectReference::cast(o)!=NULL) return true;
 
-    if ( o->getId() == getAnyElementId()) return true;
-    if (Interface::cast(o)!=NULL) return checkItfChildOfThisFw(o);
+    if (o->getId() == getAnyElementId()) return true;
+
+    if (Interface::cast(o)!=NULL) return true;
 
     if (ObjectGroup::cast(o)!=NULL)
     {
@@ -340,29 +334,37 @@ bool RuleElementItf::validateChild(FWObject *o)
  * clipboard using 'cut', it won't validate because the original has
  * lost its parent.
  * 
+ * After implementing undo(), we now run into the same problem with
+ * the rule element object because the GUI tries to call addRef()
+ * using a copy of the RuleElement object (the copy is a part of the
+ * undo command) and this copy does not have any parent either. The
+ * trick with using object Id does not work in this case though
+ * because the copy of RuleElement stored in the command has different
+ * ID. Will call this function from the GUI to do additional check
+ * where appropriate, but validateChild() will accept any Interface
+ * object.
  */
 bool RuleElementItf::checkItfChildOfThisFw(FWObject *o)
 {
-    FWObject* o_tmp = getRoot()->findInIndex(o->getId());
-    FWObject* o_tmp2 = this;
-
-    while (o_tmp)
+    if (Group::cast(o) != NULL)
     {
-        if (Firewall::cast(o_tmp))
+        for (FWObject::iterator i=o->begin(); i!=o->end(); ++i)
         {
-            // Check if the Firewall the Interface belongs to is the same this
-            // RuleElement belongs to
-            while (o_tmp2)
-            {
-                if (Firewall::cast(o_tmp2) && o_tmp->getId()==o_tmp2->getId())
-                    return true;
-                o_tmp2 = o_tmp2->getParent();
-            }
-            return false;
+            FWObject *o1 = FWReference::getObject(*i);
+            if (!checkItfChildOfThisFw(o1)) return false;
         }
-        o_tmp = o_tmp->getParent();
+        return true;
     }
-    return false;
+
+    FWObject* o_tmp = getRoot()->findInIndex(o->getId());
+    FWObject* o_tmp2 = getRoot()->findInIndex(this->getId());
+
+    FWObject *fw1 = o_tmp;
+    while (fw1 && Firewall::cast(fw1) == NULL) fw1 = fw1->getParent();
+    FWObject *fw2 = o_tmp2;
+    while (fw2 && Firewall::cast(fw2) == NULL) fw2 = fw2->getParent();
+
+    return (fw1 != NULL && fw1 == fw2);
 }
 
 
@@ -375,16 +377,15 @@ RuleElementOSrc::RuleElementOSrc(const FWObjectDatabase *root,bool prepopulate)
         _initialize(root);
 }
 
-int RuleElementOSrc::getAnyElementId() {
+int RuleElementOSrc::getAnyElementId() const
+{
     return FWObjectDatabase::ANY_ADDRESS_ID;
 }
 
 
 xmlNodePtr RuleElementOSrc::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -409,16 +410,15 @@ RuleElementODst::RuleElementODst(const FWObjectDatabase *root,bool prepopulate)
         _initialize(root);
 }
 
-int RuleElementODst::getAnyElementId() {
+int RuleElementODst::getAnyElementId() const
+{
     return FWObjectDatabase::ANY_ADDRESS_ID;
 }
 
 
 xmlNodePtr RuleElementODst::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -443,7 +443,7 @@ RuleElementOSrv::RuleElementOSrv(const FWObjectDatabase *root,bool prepopulate)
         _initialize(root);
 }
 
-int RuleElementOSrv::getAnyElementId()
+int RuleElementOSrv::getAnyElementId() const
 {
     return FWObjectDatabase::ANY_SERVICE_ID;
 }
@@ -451,9 +451,7 @@ int RuleElementOSrv::getAnyElementId()
 
 xmlNodePtr RuleElementOSrv::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -480,7 +478,7 @@ RuleElementTSrc::RuleElementTSrc(const FWObjectDatabase *root,bool prepopulate)
         _initialize(root);
 }
 
-int RuleElementTSrc::getAnyElementId()
+int RuleElementTSrc::getAnyElementId() const
 {
     return FWObjectDatabase::ANY_ADDRESS_ID;
 }
@@ -488,9 +486,7 @@ int RuleElementTSrc::getAnyElementId()
 
 xmlNodePtr RuleElementTSrc::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -515,16 +511,15 @@ RuleElementTDst::RuleElementTDst(const FWObjectDatabase *root,bool prepopulate)
         _initialize(root);
 }
 
-int RuleElementTDst::getAnyElementId() {
+int RuleElementTDst::getAnyElementId() const
+{
     return FWObjectDatabase::ANY_ADDRESS_ID;
 }
 
 
 xmlNodePtr RuleElementTDst::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -549,16 +544,15 @@ RuleElementTSrv::RuleElementTSrv(const FWObjectDatabase *root,bool prepopulate)
         _initialize(root);
 }
 
-int RuleElementTSrv::getAnyElementId() {
+int RuleElementTSrv::getAnyElementId() const
+{
     return FWObjectDatabase::ANY_SERVICE_ID;
 }
 
 
 xmlNodePtr RuleElementTSrv::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -597,16 +591,15 @@ RuleElementInterval::RuleElementInterval(const FWObjectDatabase *root,bool prepo
         _initialize(root);
 }
 
-int RuleElementInterval::getAnyElementId() {
+int RuleElementInterval::getAnyElementId() const
+{
     return FWObjectDatabase::ANY_INTERVAL_ID;
 }
 
 
 xmlNodePtr RuleElementInterval::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -632,15 +625,14 @@ RuleElementRDst::RuleElementRDst(const FWObjectDatabase *root,bool prepopulate)
         _initialize(root);
 }
 
-int RuleElementRDst::getAnyElementId() {
+int RuleElementRDst::getAnyElementId() const
+{
     return FWObjectDatabase::ANY_ADDRESS_ID;
 }
 
 xmlNodePtr RuleElementRDst::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -665,15 +657,14 @@ RuleElementRGtw::RuleElementRGtw(const FWObjectDatabase *root,bool prepopulate)
         _initialize(root);
 }
 
-int RuleElementRGtw::getAnyElementId() {
+int RuleElementRGtw::getAnyElementId() const
+{
     return FWObjectDatabase::ANY_ADDRESS_ID;
 }
 
 xmlNodePtr RuleElementRGtw::toXML(xmlNodePtr parent) throw(FWException)
 {
-    xmlNodePtr me = FWObject::toXML(parent, false);
-    xmlNewProp(me, TOXMLCAST("neg"),
-               TOXMLCAST(((getNeg()) ? "True" : "False")));
+    xmlNodePtr me = RuleElement::toXML(parent);
 
     for(list<FWObject*>::const_iterator j=begin(); j!=end(); ++j) 
         (*j)->toXML(me);
@@ -721,11 +712,6 @@ RuleElementRItf::RuleElementRItf()  {}
 RuleElementRItf::RuleElementRItf(const FWObjectDatabase *root,bool prepopulate) :
     RuleElementItf(root,prepopulate) {}
 
-/**
- * Unlike RuleElementItf, RuleElementRItf can hold only single object.
- * Both RuleElementItf and RuleElementRItf can only hold Interface objects
- * which must belong to the same firewall the rule element belongs to.
- */
 bool RuleElementRItf::validateChild(FWObject *o)
 {
     if (FWObjectReference::cast(o)!=NULL) return true;
@@ -733,8 +719,7 @@ bool RuleElementRItf::validateChild(FWObject *o)
     if (getChildrenCount() > 0 && !isAny()) return false;
 
     if ( o->getId() == getAnyElementId()) return true;
-    if (Interface::cast(o)!=NULL) return checkItfChildOfThisFw(o);
 
-    return false;
+    return (Interface::cast(o)!=NULL);
 }
 
