@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ServiceGroup.cpp 272 2009-04-21 04:07:15Z vadim $
+  $Id: ServiceGroup.cpp 488 2009-12-12 04:46:26Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -36,6 +36,16 @@
 #include <fwbuilder/Address.h>
 #include <fwbuilder/Interval.h>
 
+#include <fwbuilder/IPService.h>
+#include <fwbuilder/ICMPService.h>
+#include <fwbuilder/ICMP6Service.h>
+#include <fwbuilder/TCPService.h>
+#include <fwbuilder/UDPService.h>
+#include <fwbuilder/CustomService.h>
+#include <fwbuilder/TagService.h>
+#include <fwbuilder/UserService.h>
+#include <fwbuilder/FWServiceReference.h>
+
 #include <iostream>
 
 using namespace std;
@@ -80,4 +90,18 @@ xmlNodePtr ServiceGroup::toXML(xmlNodePtr parent) throw(FWException)
     return me;
 }
 
+void ServiceGroup::getAllowedTypesOfChildren(std::list<std::string> &types_list)
+{
+    types_list.clear();
+    types_list.push_back(IPService::TYPENAME);
+    types_list.push_back(ICMPService::TYPENAME);
+    types_list.push_back(ICMP6Service::TYPENAME);
+    types_list.push_back(TCPService::TYPENAME);
+    types_list.push_back(UDPService::TYPENAME);
+    types_list.push_back(CustomService::TYPENAME);
+    types_list.push_back(TagService::TYPENAME);
+    types_list.push_back(UserService::TYPENAME);
+    types_list.push_back(FWServiceReference::TYPENAME);
+}
+
 
