@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ServiceGroup.h 236 2008-12-25 20:36:41Z vadim $
+  $Id: ServiceGroup.h 534 2010-02-07 23:25:06Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -43,6 +43,8 @@ class ServiceGroup : public Group
 
     DECLARE_FWOBJECT_SUBTYPE(ServiceGroup);
 
+    DECLARE_DISPATCH_METHODS(ServiceGroup);
+    
     virtual xmlNodePtr toXML(xmlNodePtr xml_parent_node) throw(FWException);
 
     /*
@@ -51,6 +53,12 @@ class ServiceGroup : public Group
     virtual bool validateChild(FWObject *o);
     virtual FWReference* createRef();
 
+    /**
+     * get the list of object type names that can be inserted into
+     * given object group. This reflects definition of the group XML
+     * element in DTD.
+     */
+    virtual void getAllowedTypesOfChildren(std::list<std::string> &types_list);
 };
 
 }
