@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: TCPService.cpp 236 2008-12-25 20:36:41Z vadim $
+  $Id: TCPService.cpp 590 2010-03-26 03:06:41Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -57,8 +57,8 @@ TCPService::TCPService(const FWObjectDatabase *root, bool prepopulate) :
 
 TCPService::~TCPService() {}
 
-string TCPService::getProtocolName()   {    return "tcp";}
-int    TCPService::getProtocolNumber() {    return 6;    }
+string TCPService::getProtocolName()         {    return "tcp";}
+int    TCPService::getProtocolNumber() const {    return 6;    }
 
 void TCPService::init()
 {
