@@ -7,7 +7,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: TagService.cpp 236 2008-12-25 20:36:41Z vadim $
+  $Id: TagService.cpp 590 2010-03-26 03:06:41Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -37,8 +37,8 @@ using namespace std;
 
 const char *TagService::TYPENAME={"TagService"};
 
-string TagService::getProtocolName()     {    return "tag_service";}
-int    TagService::getProtocolNumber()   {    return 65001; }
+string TagService::getProtocolName()           {    return "tag_service";}
+int    TagService::getProtocolNumber() const   {    return 65001; }
 
 TagService::TagService() 
 {
