@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Tools.cpp 234 2008-12-23 06:17:53Z vadim $
+  $Id: Tools.cpp 580 2010-03-09 12:54:25Z rueegsegger $
 
 
   This program is free software which we release under the GNU General Public
@@ -238,4 +238,56 @@ unsigned int cxx_sleep(unsigned int seconds)
 #endif
 }
 
+void tokenize(const string& str, vector<string>& tokens, const string& delimiters)
+{
+    // Skip delimiters at beginning.
+    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
+    // Find first "non-delimiter".
+    string::size_type pos     = str.find_first_of(delimiters, lastPos);
+
+    while (string::npos != pos || string::npos != lastPos)
+    {
+        // Found a token, add it to the vector.
+        tokens.push_back(str.substr(lastPos, pos - lastPos));
+        // Skip delimiters.  Note the "not_of"
+        lastPos = str.find_first_not_of(delimiters, pos);
+        // Find next "non-delimiter"
+        pos = str.find_first_of(delimiters, lastPos);
+    }
+}
+
+string strip(const string& in, const string& identifier)
+{
+    string result;
+    vector<string> parts;
+
+    tokenize(in, parts, identifier);
+
+    for (vector<string>::iterator it = parts.begin(); it != parts.end(); ++it)
+    {
+        result += *it;
+    }
+
+    return result;
+}
+
+string stringify(const vector<string>& parts, const string& delimiter)
+{
+    string result;
+
+    vector<string>::const_iterator it = parts.begin();
+
+    while (it != parts.end())
+    {
+        result += *it++;
+
+        if (it != parts.end())
+        {
+            result += delimiter;
+        }
+    }
+
+    return result;
+}
+
 }
