@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: UDPService.cpp 236 2008-12-25 20:36:41Z vadim $
+  $Id: UDPService.cpp 590 2010-03-26 03:06:41Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -42,6 +42,6 @@ UDPService::UDPService(const FWObjectDatabase *root, bool prepopulate) :
 
 UDPService::~UDPService() {}
 
-string UDPService::getProtocolName()   {    return "udp"; }
-int    UDPService::getProtocolNumber() {    return 17;    }
+string UDPService::getProtocolName()         {    return "udp"; }
+int    UDPService::getProtocolNumber() const {    return 17;    }
 
