@@ -4,9 +4,9 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: UDPService.h 105 2008-06-09 01:52:12Z vadim $
+  $Id: UDPService.h 590 2010-03-26 03:06:41Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -38,13 +38,15 @@ class UDPService : public TCPUDPService
     public:
 
     UDPService();
-    UDPService(const FWObject *root,bool prepopulate);
+    UDPService(const FWObjectDatabase *root,bool prepopulate);
     virtual ~UDPService();
     
     DECLARE_FWOBJECT_SUBTYPE(UDPService);
+
+    DECLARE_DISPATCH_METHODS(UDPService);
     
     virtual std::string getProtocolName();
-    virtual int    getProtocolNumber();
+    virtual int getProtocolNumber() const;
     
 };
     
