@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: XMLTools.cpp 329 2009-05-27 20:43:05Z vadim $
+  $Id: XMLTools.cpp 583 2010-03-16 17:03:48Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -57,6 +57,9 @@
 #include <libxslt/xsltInternals.h>
 #include <libxslt/transform.h>
 #include <libxslt/xsltutils.h>
+
+#include <libxml/xmlmemory.h>
+
 #include <zlib.h>
 
 #include <iostream>
@@ -213,16 +216,21 @@ void XMLTools::initXMLTools()
 {
 //    xml_parser_mutex     = PTHREAD_MUTEX_INITIALIZER;
 //    xslt_processor_mutex = PTHREAD_MUTEX_INITIALIZER;
+    xmlInitMemory();
+    xmlInitParser();
     defaultLoader = xmlGetExternalEntityLoader();
     current_template_dir=cxx_strdup("");
     xmlSetExternalEntityLoader(fwbExternalEntityLoader);
 }
 
+void XMLTools::close()
+{
+    xmlCleanupParser();
+}
+
 string XMLTools::readFile(const std::string &rfile) throw(FWException)
 {
     string buf;
-    struct stat stt;
-    int         fd;
 
     if (rfile=="-")
     {
@@ -316,9 +324,27 @@ xmlDocPtr XMLTools::loadFile(const string &data_file ,
          << "    current_version: " << current_version
          << endl;
 #endif
-    
-    if (data_file!="-" && access(data_file.c_str() , R_OK )!=0)
-        throw FWException(string("Could not access data file: ")+data_file);
+    int access_err = 0;
+    if (data_file!="-" && (access_err=access(data_file.c_str() , R_OK ))!=0)
+    {
+        string access_err_str;
+        switch (access_err)
+        {
+        case EACCES: access_err_str = "EACCES"; break;
+        case EFAULT: access_err_str = "EFAULT"; break;
+        case EIO: access_err_str = "EIO"; break;
+#ifdef ELOOP
+        case ELOOP: access_err_str = "ELOOP"; break;
+#endif
+        case ENAMETOOLONG: access_err_str = "ENAMETOOLONG"; break;
+        case ENOENT: access_err_str = "ENOENT"; break;
+        case ENOTDIR: access_err_str = "ENOTDIR"; break;
+        default: access_err_str = "Unknown"; break;
+        }
+        throw FWException(
+            string("Could not access data file '") + data_file + "'" +
+            " error code " + access_err_str);
+    }
 
     string buf = readFile(data_file);
 
@@ -353,7 +379,7 @@ in the same directory with extension '.bak'. Are you sure you want to open it?";
         if(!(*upgrade)(upgrade_msg))
         {
             xmlFreeDoc(newdoc);
-            xmlCleanupParser();
+            //xmlCleanupParser();
             throw FWException("Load operation cancelled for file: '"+data_file);
         }
      
@@ -370,7 +396,7 @@ in the same directory with extension '.bak'. Are you sure you want to open it?";
         if(rename(data_file.c_str(), backup_file.c_str()))
         {
             xmlFreeDoc(doc);
-            xmlCleanupParser();
+            //xmlCleanupParser();
             throw FWException("Error making backup copy of file: '" + 
                               data_file + "' as '" + backup_file + "'");
         }
@@ -394,7 +420,7 @@ in the same directory with extension '.bak'. Are you sure you want to open it?";
     } 
     assert(doc!=NULL);
     xmlFreeDoc(doc);
-    xmlCleanupParser();
+    //xmlCleanupParser();
 
     // Now we know the version is OK,
     // let us load for real, checking DTD.
@@ -568,7 +594,7 @@ void XMLTools::transformFileToFile(const string &src_file,
     xmlFreeDoc(doc);
 
     xsltCleanupGlobals();
-    xmlCleanupParser();
+    //xmlCleanupParser();
 }
 
 void XMLTools::transformDocumentToFile(xmlDocPtr doc, 
@@ -646,7 +672,7 @@ void XMLTools::transformDocumentToFile(xmlDocPtr doc,
         xsltSaveResultToFilename(dst_file.c_str(), res, ss, 0 /* compression */ );
 
     xmlFreeDoc(res);
-    xmlCleanupParser();
+    //xmlCleanupParser();
     xsltFreeStylesheet(ss);
 }
 
@@ -723,7 +749,7 @@ xmlDocPtr XMLTools::convert(xmlDocPtr doc,
     if (!root || !root->name || type_name!=FROMXMLCAST(root->name))
     {
         xmlFreeDoc(doc);
-        xmlCleanupParser();
+        //xmlCleanupParser();
         throw FWException("XML file '"+file_name+ "' has invalid structure.");
     }
 
@@ -780,7 +806,7 @@ xmlDocPtr XMLTools::convert(xmlDocPtr doc,
         if (access(fname.c_str() , R_OK )!=0) 
         {
             xmlFreeDoc(doc);
-            xmlCleanupParser();
+            //xmlCleanupParser();
             throw FWException(
                 string("File '" + file_name +
                        "' conversion error: no converter found for version: ") +
@@ -794,18 +820,18 @@ xmlDocPtr XMLTools::convert(xmlDocPtr doc,
         {
             ex.getProperties()["failed_transformation"]=fname;
             xmlFreeDoc(doc);
-            xmlCleanupParser();
+            //xmlCleanupParser();
             throw;
         }
         xmlFreeDoc(doc);
-        xmlCleanupParser();
+        //xmlCleanupParser();
         doc = res;
         
         root = xmlDocGetRootElement(doc);
         if (!root || !root->name || type_name!=FROMXMLCAST(root->name))
         {
             xmlFreeDoc(doc);
-            xmlCleanupParser();
+            //xmlCleanupParser();
             throw FWException("File '" + file_name +
                               "' conversion Error: conversion produced file with invalid structure.");
         }
@@ -814,7 +840,7 @@ xmlDocPtr XMLTools::convert(xmlDocPtr doc,
         if (v==NULL)
         {
             xmlFreeDoc(doc);
-            xmlCleanupParser();
+            //xmlCleanupParser();
             throw FWException("File '" + file_name +
                               "' conversion error: converted to unknown version.");
         } 
@@ -825,7 +851,7 @@ xmlDocPtr XMLTools::convert(xmlDocPtr doc,
         if (version_compare(vers, oldversion) <= 0)
         {
             xmlFreeDoc(doc);
-            xmlCleanupParser();
+            //xmlCleanupParser();
             throw FWException("File '" + file_name +
                               "' conversion error: conversion did not advance version number!.");
         }
