@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: snmp.cpp 243 2009-01-10 23:58:22Z vadim $
+  $Id: snmp.cpp 612 2010-06-08 20:43:20Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -503,7 +503,7 @@ void SNMPQuery::walkInterfaceIndexTable(Logger *logger,
     w.clear();
 }
 
-void SNMPQuery::getAddressAndNetmask(Logger *logger,
+void SNMPQuery::getAddressAndNetmask(Logger * /* UNUSED logger */,
                                      SNMPConnection *c,
                                      std::string adentry,
                                      InetAddr **addr,
@@ -686,12 +686,6 @@ void SNMPQuery::fetchInterfaces(Logger *logger, SyncFlag *stop_program,
             SNMPVariable::freeVarList(v);
 
             list<string> &addlist = addr[ifindex];
-            if (addlist.empty())
-            {
-                str << "Interface #" << ifindex << " " << descr
-                    << " has no IP address.\n";
-                *logger << str;
-            }
 
             // Get physical address
             sprintf(oid, "%s.%ld", SNMP_INTERFACES_PHYSA , ifindex);
@@ -721,6 +715,17 @@ void SNMPQuery::fetchInterfaces(Logger *logger, SyncFlag *stop_program,
 
             if (physa!="") idata.mac_addr = physa;
 
+            str << "Adding interface #" << ifindex 
+                << " " << idata.name
+                << " MAC: " << idata.mac_addr << endl;
+            *logger << str;
+
+             if (addlist.empty())
+             {
+                 str << "    no IP addresses." << endl;
+                 *logger << str;
+             }
+
             for (list<string>::iterator ali = addlist.begin();
                  ali!=addlist.end(); ali++)
             {
@@ -738,8 +743,8 @@ void SNMPQuery::fetchInterfaces(Logger *logger, SyncFlag *stop_program,
                 iam->setNetmask(*nm);
                 idata.addr_mask.push_back(iam);
 
-                str << "Adding interface #" << ifindex 
-                    << " " << descr
+                str << "    " << descr
+                    << " IP address"
                     << " " << iam->toString()
                     << "\n";
                 *logger << str;
@@ -1260,7 +1265,6 @@ SNMPCrawler::SNMPCrawler()
 SNMPCrawler::SNMPCrawler(const InetAddr &_seed, 
                          const string &_community,
                          bool _recursive,
-                         bool _skip_virtual,
                          bool _do_dns,
                          bool _follow_ptp,
                          unsigned int _dns_threads,
@@ -1270,7 +1274,7 @@ SNMPCrawler::SNMPCrawler(const InetAddr &_seed,
 			 int  _dns_timeout,
                          const vector<InetAddrMask> *_include)
 {
-    init(_seed, _community, _recursive, _skip_virtual, 
+    init(_seed, _community, _recursive, 
          _do_dns, _follow_ptp, _dns_threads, 
 	 _snmp_retries,  _snmp_timeout, _dns_retries, _dns_timeout, 
 	 _include);
@@ -1283,7 +1287,6 @@ SNMPCrawler::~SNMPCrawler()
 void SNMPCrawler::init(const InetAddr &_seed, 
 		       const string &_community,
 		       bool _recursive,
-                       bool _skip_virtual,
                        bool _do_dns,
                        bool _follow_ptp,
                        unsigned int _dns_threads,
@@ -1298,7 +1301,6 @@ void SNMPCrawler::init(const InetAddr &_seed,
     snmp_retries = _snmp_retries;
     snmp_timeout = _snmp_timeout;
     recursive    = _recursive;
-    skip_virtual = _skip_virtual;
     do_dns       = _do_dns;
     follow_ptp   = _follow_ptp;
     dns_threads  = _dns_threads;
@@ -1370,43 +1372,6 @@ const InetAddrMask SNMPCrawler::IPV6_LOOPBACK_NET(
 
 const InetAddr SNMPCrawler::PTP_NETMASK(InetAddr::getAllOnes());
 
-bool SNMPCrawler::isvirtual(const InetAddr&, const string&) const 
-{
-// 01/10/2009 : this code is very old. We used to explicitly check and skip
-// virtual interfaces, however on Linux at least virtual interfaces
-// are just secondary ip addresses that belong to normal interfaces and
-// should not be skipped.
-#if 0
-    if (!pa.length()) return false;
-    for (map<InetAddr, CrawlerFind>::const_iterator i=found.begin();
-         i!=found.end(); ++i)
-    {
-        const CrawlerFind &c = i->second;
-        
-        map<int, InterfaceData>::const_iterator j;
-        for(j=c.interfaces.begin(); j!=c.interfaces.end(); ++j)
-        {
-            if (!j->second.addr_mask) continue; // no ip addr.
-            try
-            {
-                const InetAddr *intf_addr =
-                    j->second.addr_mask->getAddressPtr();
-
-                string paddr = j->second.mac_addr;
-                if (paddr!="" && pa == paddr &&
-                    intf_addr!=NULL && addr != *intf_addr)
-                    return true;
-            } catch(FWException &ex)
-            {
-                // Exception caused by interface having bad IP address.
-                // we just skip this interface and go on
-            }
-        }
-    }
-#endif    
-    return false;
-}
-
 bool SNMPCrawler::point2point(const InetAddrMask &n,
                               const InterfaceData& intf) const
 {
@@ -1529,8 +1494,7 @@ void SNMPCrawler::run_impl(Logger *logger,
 
             InetAddr  c = (*j).first;
             string    pa = (*j).second;
-            if (included(c) && !alreadyseen(c) &&
-                !isvirtual(c,pa) && !special(c))
+            if (included(c) && !alreadyseen(c) && !special(c))
             {
                 if(recursive)
                 {
@@ -1736,8 +1700,7 @@ void SNMPCrawler::run_impl(Logger *logger,
                         // for some of our interfaces, and if yes, ignore them.
                         // (see task #36520).
 
-                        if (included(*c) && !alreadyseen(*c) && 
-                            !isvirtual(*c,"") && !special(*c) && 
+                        if (included(*c) && !alreadyseen(*c) && !special(*c) && 
                             !interface_broadcasts.count(*c))
                         {
                             if(recursive && follow_ptp)
@@ -1770,7 +1733,7 @@ void SNMPCrawler::run_impl(Logger *logger,
                 }
 
                 InetAddr gw((*j).getGateway());
-                if (included(gw) && !alreadyseen(gw) && !isvirtual(gw, "") &&
+                if (included(gw) && !alreadyseen(gw) && 
                     !special(gw) && !interface_broadcasts.count(gw))
                 {
                     bool isptp=point2point(net, intf);
@@ -1828,9 +1791,6 @@ void SNMPCrawler::run_impl(Logger *logger,
         
     } while(recursive);
 
-    if(skip_virtual)
-        remove_virtual(logger,stop_program);
-    
     if(do_dns)
         bacresolve_results(logger,stop_program);
     
@@ -1841,14 +1801,6 @@ void SNMPCrawler::run_impl(Logger *logger,
     *logger << str;
 }
 
-void SNMPCrawler::remove_virtual(Logger *logger, SyncFlag*) throw(FWException)
-{
-    *logger << "Removing virtual IPs.\n";
-    for(map<InetAddr, CrawlerFind>::iterator j=found.begin(); j!=found.end(); ++j)
-        if(isvirtual((*j).first, (*j).second.found_phys_addr))
-            found.erase(j);
-}
-
 void SNMPCrawler::bacresolve_results(Logger *logger,
                                      SyncFlag *) throw(FWException)
 {
