@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Compiler.cpp 353 2009-06-08 21:33:04Z vadim $
+  $Id: Compiler.cpp 604 2010-05-12 21:10:31Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -33,6 +33,7 @@
 #include "fwbuilder/FWObjectReference.h"
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/RuleElement.h"
+#include "fwbuilder/Cluster.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Network.h"
 #include "fwbuilder/NetworkIPv6.h"
@@ -44,12 +45,13 @@
 #include "fwbuilder/CustomService.h"
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/Rule.h"
+#include "fwbuilder/RuleSet.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/IPv6.h"
 #include "fwbuilder/DNSName.h"
 #include "fwbuilder/MultiAddress.h"
-
+#include "fwbuilder/FailoverClusterGroup.h"
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/XMLTools.h"
 #include "fwbuilder/FWException.h"
@@ -67,63 +69,51 @@ using namespace libfwbuilder;
 using namespace fwcompiler;
 using namespace std;
 
-FWCompilerException::FWCompilerException(Rule *r,const string &err) : FWException(err)
-{
-    rule=r;
-}
-
 
 Compiler::~Compiler() {}
 
-int Compiler::cache_objects(FWObject *o)
-{
-    if ( o->getId() > -1 )  cacheObj(o);
-
-    int n=0;
-    for (FWObject::iterator i=o->begin(); i!=o->end(); ++i) {
-        n=n+1+cache_objects((*i));
-    }
-    return n;
-}
-
-void Compiler::cacheObj(libfwbuilder::FWObject *o)
-{
-    objcache[o->getId()]=o;
-}
-
-
 int Compiler::prolog() 
 {
-    temp=new Group();
-
-    fw->add(temp,false);
-
-/* caching firewall interfaces */
-    FWObjectTypedChildIterator j=fw->findByType(Interface::TYPENAME);
-    for ( ; j!=j.end(); ++j )
-    {
-        Interface *interface_ = Interface::cast(*j);
-        fw_interfaces[interface_->getId()] = interface_;
-    }
-    
-    fw_id=fw->getId();
-
-    fwopt = fw->getOptionsObject();
-
-/* caching all objects */
-
-    cache_objects( dbcopy );
-
+    temp = new Group();
+    fw->add(temp, false);
     return 0;
 }
 
 void Compiler::epilog()
 {
-    cerr << "Compiler::epilog must be overloaded\n";
-    exit(1);
 }
 
-int    Compiler::getCompiledScriptLength()
+void Compiler::abort(const string &errstr) throw(FWException)
+{
+    BaseCompiler::abort(fw, source_ruleset, NULL, errstr);
+}
+
+void Compiler::abort(FWObject *rule, const string &errstr) throw(FWException)
+{
+    BaseCompiler::abort(fw, source_ruleset, rule, errstr);
+}
+
+void Compiler::error(const string &errstr)
+{
+    BaseCompiler::error(fw, source_ruleset, NULL, errstr);
+}
+
+void Compiler::error(FWObject *rule, const string &errstr)
+{
+    BaseCompiler::error(fw, source_ruleset, rule, errstr);
+}
+
+void Compiler::warning(const string &errstr)
+{
+    BaseCompiler::warning(fw, source_ruleset, NULL, errstr);
+}
+
+void Compiler::warning(FWObject *rule, const string &errstr)
+{
+    BaseCompiler::warning(fw, source_ruleset, rule, errstr);
+}
+
+int Compiler::getCompiledScriptLength()
 {
     return int(output.tellp());
 }
@@ -145,71 +135,52 @@ string Compiler::getCompiledScript()
     return res;
 }
 
-bool Compiler::haveErrorsAndWarnings()
-{
-    return (int(output.tellp()) > 0);
-}
-
-string Compiler::getErrors(const string &comment_sep)
-{
-    ostringstream ostr;
-    istringstream istr(errors_buffer.str());
-    while (!istr.eof())
-    {
-        string tmpstr;
-        getline(istr, tmpstr);
-        ostr << comment_sep << tmpstr << endl;
-    }
-    errors_buffer.str("");
-    return ostr.str();
-}
-
-void Compiler::_init(FWObjectDatabase *_db, const string &fwobjectname)
+void Compiler::_init(FWObjectDatabase *_db, Firewall *_fw)
 { 
-    initialized=false;
-    _cntr_=1; 
+    initialized = false;
+    _cntr_ = 1; 
 
-    fw=NULL; 
-    temp_ruleset=NULL; 
-    combined_ruleset=NULL;
+    temp_ruleset = NULL; 
+    combined_ruleset = NULL;
 
-    debug=0;
-    debug_rule=-1;
-    verbose=true;
+    debug = 0;
+    debug_rule = -1;
+    rule_debug_on = false;
+    verbose = true;
+    single_rule_mode = false;
+    single_rule_ruleset_name = "";
+    single_rule_position = -1;
+
+    fw_id = _fw->getId();
+    fwopt = _fw->getOptionsObject();
+
+    assert(_fw->getRoot() == _db);
+
+    string fw_str_id = FWObjectDatabase::getStringId(_fw->getId());
     
-    dbcopy=new FWObjectDatabase(*_db);  // copies entire tree
-
-    fw=dbcopy->findFirewallByName(fwobjectname);
-    if (fw==NULL) {
-	cerr << "Firewall object '" << fwobjectname << "' not found \n";
-	exit(1);
-    }
-
+    dbcopy = new FWObjectDatabase(*_db);  // copies entire tree
+    fw = Firewall::cast(dbcopy->findInIndex(FWObjectDatabase::getIntId(fw_str_id)));
 }
 
-Compiler::Compiler(FWObjectDatabase *_db, 
-		   const string &fwobjectname, bool ipv6_policy)
+Compiler::Compiler(FWObjectDatabase *_db, Firewall *fw, bool ipv6_policy)
 {
     source_ruleset = NULL;
     ruleSetName = "";
-    test_mode = false;
     osconfigurator = NULL;
     countIPv6Rules = 0;
     ipv6 = ipv6_policy;
-    _init(_db,fwobjectname);
+    _init(_db, fw);
 }
 
-Compiler::Compiler(FWObjectDatabase *_db, 
-		   const string &fwobjectname, bool ipv6_policy,
+Compiler::Compiler(FWObjectDatabase *_db, Firewall *fw, bool ipv6_policy,
 		   OSConfigurator *_oscnf)
 {
     source_ruleset = NULL;
     ruleSetName = "";
-    test_mode = false;
     osconfigurator = _oscnf;
     countIPv6Rules = 0;
     ipv6 = ipv6_policy;
-    _init(_db,fwobjectname);
+    _init(_db, fw);
 }
 
 // this constructor is used by class Preprocessor, it does not call _init
@@ -217,7 +188,6 @@ Compiler::Compiler(FWObjectDatabase*, bool ipv6_policy)
 {
     source_ruleset = NULL;
     ruleSetName = "";
-    test_mode = false;
     osconfigurator = NULL;
     countIPv6Rules = 0;
     ipv6 = ipv6_policy;
@@ -228,7 +198,25 @@ Compiler::Compiler(FWObjectDatabase*, bool ipv6_policy)
     combined_ruleset = NULL;
     debug = 0;
     debug_rule = -1;
+    rule_debug_on = false;
     verbose = true;
+    single_rule_mode = false;
+}
+
+void Compiler::setSingleRuleCompileMode(const string &rule_id)
+{
+    if (!rule_id.empty())
+    {
+        Rule *rule = Rule::cast(
+            dbcopy->findInIndex(FWObjectDatabase::getIntId(rule_id)));
+        if (rule)
+        {
+            single_rule_mode = true;
+            single_rule_compile_rule = rule;
+            single_rule_position = rule->getPosition();
+            single_rule_ruleset_name = rule->getParent()->getName();
+        }
+    }
 }
 
 string Compiler::createRuleLabel(const std::string &prefix,
@@ -242,32 +230,6 @@ string Compiler::createRuleLabel(const std::string &prefix,
     return str.str();
 }
 
-void Compiler::abort(const string &errstr) throw(FWException)
-{
-    if (test_mode)
-        error(errstr);
-    else
-        throw FWException(errstr);
-}
-
-void Compiler::error(const string &errstr)
-{
-    cout << flush;
-    cerr << "Error (" << myPlatformName() << "): ";
-    cerr << errstr << endl;
-    errors_buffer << "Error (" << myPlatformName() << "): ";
-    errors_buffer << errstr << endl;
-}
-
-void Compiler::warning(const string &warnstr)
-{
-    cout << flush;
-    cerr << "Warning (" << myPlatformName() << "): ";
-    cerr << warnstr << endl;
-    errors_buffer << "Warning (" << myPlatformName() << "): ";
-    errors_buffer << warnstr << endl;
-}
-
 string Compiler::getUniqueRuleLabel()
 {
     ostringstream str;
@@ -285,6 +247,13 @@ void Compiler::compile()
 
 void Compiler::_expand_group_recursive(FWObject *o, list<FWObject*> &ol)
 {
+/*
+ * ref #50: ignore various Options child objects. In particular this
+ * skips ClusterGroupOptions object which is a child of
+ * FailoverClusterGroup and StateSyncClusterGroup objects.
+ */
+    if (FWOptions::cast(o)) return;
+
 /* special case: MultiAddress. This class inherits ObjectGroup, but
  * should not be expanded if it is expanded at run time
  *
@@ -352,9 +321,10 @@ void Compiler::expandGroupsInRuleElement(RuleElement *s)
 }
 
 void Compiler::_expand_addr_recursive(Rule *rule, FWObject *s,
-                                      list<FWObject*> &ol)
+                                      list<FWObject*> &ol,
+                                      bool expand_cluster_interfaces_fully)
 {
-    Interface *rule_iface = fw_interfaces[rule->getInterfaceId()];
+    Interface *rule_iface = Interface::cast(dbcopy->findInIndex(rule->getInterfaceId()));
     bool on_loopback= ( rule_iface && rule_iface->isLoopback() );
 
     list<FWObject*> addrlist;
@@ -389,7 +359,7 @@ void Compiler::_expand_addr_recursive(Rule *rule, FWObject *s,
             continue;
         }
     }
-
+ 
     if (addrlist.empty())
     {
         if (RuleElement::cast(s)==NULL) ol.push_back(s);
@@ -402,6 +372,11 @@ void Compiler::_expand_addr_recursive(Rule *rule, FWObject *s,
             Interface *i2itf = Interface::cast(*i2);
             if (i2itf)
             {
+/*
+ * skip copy of the member interface added in CompilerDriver::copyFailoverInterface
+ */
+                if (i2itf->getBool("member_interface_copy")) continue;
+                    
 /*
  * Special case is loopback interface - skip it, but only if this rule is
  * not attached to loopback!
@@ -414,25 +389,23 @@ void Compiler::_expand_addr_recursive(Rule *rule, FWObject *s,
                 if (i2itf->isLoopback())
                 {
                     if (RuleElement::cast(s) || on_loopback)
-                        _expandInterface(i2itf, ol);
+                        _expand_interface(rule, i2itf, ol, expand_cluster_interfaces_fully);
                 } else
 // this is not a loopback interface                
-                    _expandInterface(i2itf, ol);
+                    _expand_interface(rule, i2itf, ol, expand_cluster_interfaces_fully);
 
                 continue;
             }
-            _expand_addr_recursive(rule, *i2, ol);
+            _expand_addr_recursive(rule, *i2, ol, expand_cluster_interfaces_fully);
         }
     }
 }
 
-void Compiler::_expandInterface(Interface *iface, std::list<FWObject*> &ol)
+void Compiler::_expand_interface(Rule *rule,
+                                 Interface *iface,
+                                 std::list<FWObject*> &ol,
+                                 bool expand_cluster_interfaces_fully)
 {
-/*
- * if this is unnumbered interface or a bridge port, then do not use it
- */
-    if (iface->isUnnumbered() || iface->isBridgePort()) return;
-
 /*
  * if this is an interface with dynamic address, then simply use it
  * (that is, do not use its children elements "Address")
@@ -443,17 +416,15 @@ void Compiler::_expandInterface(Interface *iface, std::list<FWObject*> &ol)
         return;
     }
 
-//    physAddress *pa=iface->getPhysicalAddress();
 /*
  * we use physAddress only if Host option "use_mac_addr_filter" of the
  * parent Host object is true
  */
-    FWObject  *p;
-    FWOptions *hopt;
-    p = iface->getParent();
+    FWObject  *p = iface->getParentHost();
     Host *hp = Host::cast(p);
-    bool use_mac= (hp && (hopt = hp->getOptionsObject())!=NULL &&
-                   hopt->getBool("use_mac_addr_filter") ); 
+    if (hp==NULL) return;  // something is very broken
+    FWOptions *hopt = hp->getOptionsObject();
+    bool use_mac = (hopt!=NULL && hopt->getBool("use_mac_addr_filter")); 
 
     for (FWObject::iterator i1=iface->begin(); i1!=iface->end(); ++i1) 
     {
@@ -465,12 +436,53 @@ void Compiler::_expandInterface(Interface *iface, std::list<FWObject*> &ol)
             continue;
         }
 
+        // Skip bridge ports
+        Interface *subint = Interface::cast(o);
+        if (subint)
+        {
+            if (subint->isBridgePort()) continue;
+            _expand_interface(rule, subint, ol, expand_cluster_interfaces_fully);
+            continue;
+        }
+
         if (Address::cast(o)!=NULL && MatchesAddressFamily(o)) ol.push_back(o);
     }
 
-//    FWObjectTypedChildIterator j=iface->findByType(IPv4::TYPENAME);
-//    for ( ; j!=j.end(); ++j ) 
-//        ol.push_back(*j);
+    if (expand_cluster_interfaces_fully && iface->isFailoverInterface())
+    {
+        // See #1234  Cluster failover interface expands to its own addresses,
+        // plus addresses of the corresponding member interface
+
+        FailoverClusterGroup *fg = FailoverClusterGroup::cast(
+            iface->getFirstByType(FailoverClusterGroup::TYPENAME));
+
+        Interface* member_intf = fg->getInterfaceForMemberFirewall(fw);
+        if (member_intf) 
+            _expand_interface(rule, member_intf, ol, expand_cluster_interfaces_fully);
+        else
+        {
+            // per #1394, if the cluster interface used in the rule does not
+            // belong to the cluster being compiled, expand it to its own
+            // address and addresses of all corresponding member interfaces
+            for (FWObjectTypedChildIterator it =
+                     fg->findByType(FWObjectReference::TYPENAME);
+                 it != it.end(); ++it)
+            {
+                Interface *other_intf = Interface::cast(FWObjectReference::getObject(*it));
+                assert(other_intf);
+                _expand_interface(rule, other_intf, ol, expand_cluster_interfaces_fully);
+            }
+        }
+    }
+}
+
+bool compare_addresses(Address *a1, Address *a2)
+{
+    const InetAddr *addr1 = a1->getAddressPtr();
+    const InetAddr *addr2 = a2->getAddressPtr();
+    if (addr1 == NULL) return true;
+    if (addr2 == NULL) return false;
+    return *addr1 < *addr2;
 }
 
 /**
@@ -480,23 +492,25 @@ void Compiler::_expandInterface(Interface *iface, std::list<FWObject*> &ol)
  * a pointer at either src or dst in the rule
  *
  */
-void Compiler::_expandAddr(Rule *rule, FWObject *s) 
+void Compiler::_expand_addr(Rule *rule, FWObject *s,
+                            bool expand_cluster_interfaces_fully) 
 {
     list<FWObject*> cl;
+    _expand_addr_recursive(rule, s, cl, expand_cluster_interfaces_fully);
+    
+    list<Address*> expanded_addresses;
+    for (FWObject::iterator i=cl.begin(); i!=cl.end(); ++i) 
+    {
+        expanded_addresses.push_back(Address::cast(*i));
+    }
 
-    _expand_addr_recursive(rule, s, cl);
+    expanded_addresses.sort(compare_addresses);
 
     s->clearChildren();
 
-    for (FWObject::iterator i1=cl.begin(); i1!=cl.end(); ++i1) 
+    for (list<Address*>::iterator i1=expanded_addresses.begin();
+         i1!=expanded_addresses.end(); ++i1) 
     {
-#if 0
-        cerr << "Compiler::_expandAddr: adding object: (*i1)->name=";
-        cerr << (*i1)->getName();
-        cerr << "  (*i1)->id=" << (*i1)->getId();
-        cerr << "  type=" << (*i1)->getTypeName();
-        cerr << endl;
-#endif        
         s->addRef( *i1 );
     }
 }
@@ -535,7 +549,6 @@ void Compiler::_expandAddressRanges(Rule*, FWObject *re)
                     h->setName(string("%n-")+(*i).toString()+string("%") );
                     h->setNetmask(*(i->getNetmaskPtr()));
                     h->setAddress(*(i->getAddressPtr()));
-                    cacheObj(h); // to keep cache consistent
                     dbcopy->add(h,false);
                     cl.push_back(h);
                 }
@@ -558,18 +571,16 @@ void Compiler::normalizePortRange(int &rs,int &re)
     if (rs!=0 && re==0) re=rs;
 }
 
-
-
 void Compiler::debugRule()
 {
     for (FWObject::iterator i=combined_ruleset->begin();
          i!=combined_ruleset->end(); i++)
     {
 	Rule *rule = Rule::cast( *i );
-        if ( rule->getPosition()==debug_rule )
+        if (rule_debug_on && rule->getPosition()==debug_rule )
         {
-            cout << debugPrintRule(rule);
-            cout << endl;
+            info(debugPrintRule(rule));
+            info("\n");
         }
     }
 }
@@ -593,7 +604,7 @@ string Compiler::debugPrintRule(libfwbuilder::Rule *rule)
 void Compiler::add(BasicRuleProcessor* rp) 
 { 
     rule_processors.push_back(rp); 
-    if (debug_rule>=0  && dynamic_cast<simplePrintProgress*>(rp)==NULL) 
+    if (rule_debug_on && dynamic_cast<simplePrintProgress*>(rp)==NULL) 
         rule_processors.push_back(new Debug());
 }
 
@@ -614,7 +625,11 @@ void Compiler::runRuleProcessors()
 
 void Compiler::deleteRuleProcessors()
 {
-
+    list<BasicRuleProcessor*>::iterator i=rule_processors.begin();
+    for ( ; i!=rule_processors.end(); ++i)
+    {
+        delete *i;
+    }
     rule_processors.clear();
 }
 
@@ -636,14 +651,16 @@ bool Compiler::Begin::processNext()
         for (FWObject::iterator i=compiler->combined_ruleset->begin();
              i!=compiler->combined_ruleset->end(); ++i)
         {
-            Rule *rule=Rule::cast(*i);
-            Rule  *r= Rule::cast(compiler->dbcopy->create(rule->getTypeName()));
+            Rule *rule = Rule::cast(*i);
+            if (rule->isDisabled()) continue;
+            Rule  *r = Rule::cast(compiler->dbcopy->create(rule->getTypeName()));
             compiler->temp_ruleset->add(r);
             r->duplicate(rule);
             tmp_queue.push_back( r );
         }
         init=true;
-        if (!name.empty()) cout << " " << name << endl << flush;
+        if (!name.empty())
+            compiler->info(string(" ") + name);
 
         return true;
     }
@@ -657,7 +674,12 @@ bool Compiler::printTotalNumberOfRules::processNext()
 
     slurp();
     if (tmp_queue.size()==0) return false;
-    if (compiler->verbose) cout << " processing " << tmp_queue.size() << " rules" << endl << flush;
+    if (compiler->verbose)
+    {
+        ostringstream str;
+        str << " processing " << tmp_queue.size() << " rules";
+        compiler->info(str.str());
+    }
     return true;
 }
 
@@ -668,7 +690,7 @@ bool Compiler::createNewCompilerPass::processNext()
 
     slurp();
     if (tmp_queue.size()==0) return false;
-    cout << pass_name << endl << flush;
+    compiler->info(pass_name);
     return true;
 }
 
@@ -680,22 +702,45 @@ bool Compiler::Debug::processNext()
     slurp();
     if (tmp_queue.size()==0) return false;
 
-    if (compiler->debug_rule>=0) {
-        string n=prev_processor->getName();
-        cout << endl << flush;
-        cout << "--- "  << n << " " << setw(74-n.length()) << setfill('-') << "-" << flush;
-
-        for (std::deque<Rule*>::iterator i=tmp_queue.begin(); i!=tmp_queue.end(); ++i) {
-            Rule *rule=Rule::cast(*i);
-            if ( rule->getPosition()==compiler->debug_rule ) {
-                cout << compiler->debugPrintRule(rule) << flush;
-                cout << endl << flush;
+    if (compiler->rule_debug_on)
+    {
+        string n = prev_processor->getName();
+        ostringstream str;
+        str << endl << "--- "  << n << " " << setw(74-n.length()) << setfill('-') << "-";
+        compiler->info(str.str());
+        for (std::deque<Rule*>::iterator i=tmp_queue.begin(); i!=tmp_queue.end(); ++i)
+        {
+            Rule *rule = Rule::cast(*i);
+            if (compiler->rule_debug_on && rule->getPosition()==compiler->debug_rule )
+            {
+                compiler->info(compiler->debugPrintRule(rule));
+                compiler->info("\n");
             }
         }
     }
     return true;
 }
 
+bool Compiler::singleRuleFilter::processNext()
+{
+    assert(compiler!=NULL);
+    assert(prev_processor!=NULL);
+
+    Rule *rule = prev_processor->getNextRule(); if (rule==NULL) return false;
+
+    if (!compiler->single_rule_mode)
+    {
+        tmp_queue.push_back(rule);
+        return true;
+    }
+
+    if (compiler->single_rule_ruleset_name == compiler->ruleSetName &&
+        rule->getPosition() == compiler->single_rule_position)
+        tmp_queue.push_back(rule);
+
+    return true;
+}
+
 bool Compiler::simplePrintProgress::processNext()
 {
     Rule *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
@@ -704,8 +749,8 @@ bool Compiler::simplePrintProgress::processNext()
     if (rl!=current_rule_label) {
             
         if (compiler->verbose) {
-            std::string s=" rule "+rl+"\n";
-            cout << s << flush;
+            std::string s=" rule "+rl;
+            compiler->info(s);
         }
 
         current_rule_label=rl;
@@ -721,7 +766,7 @@ bool Compiler::convertInterfaceIdToStr::processNext()
 
     if (rule->getInterfaceStr().empty())
     {
-        Interface *iface = compiler->getCachedFwInterface(rule->getInterfaceId());
+        FWObject *iface = compiler->dbcopy->findInIndex(rule->getInterfaceId());
         string iface_name= (iface!=NULL) ? iface->getName() : "";
         rule->setInterfaceStr( iface_name );
     } else
@@ -746,13 +791,15 @@ bool Compiler::splitIfRuleElementMatchesFW::processNext()
 
     list<FWObject*> cl;
 
-    for (list<FWObject*>::iterator i1=re->begin(); nre>1 && i1!=re->end(); ++i1) {
-
+    for (list<FWObject*>::iterator i1=re->begin(); nre>1 && i1!=re->end(); ++i1)
+    {
         FWObject *obj = FWReference::getObject(*i1);
         Address *a = Address::cast(obj);
         assert(a!=NULL);
 
-        if (compiler->complexMatch(a, compiler->fw))
+        if (a->getId() == compiler->fw->getId() ||
+            a->getInt("parent_cluster_id") == compiler->fw->getId() ||
+            compiler->complexMatch(a, compiler->fw))
         {
 	    cl.push_back(*i1);
             nre--;
@@ -828,7 +875,9 @@ void  Compiler::recursiveGroupsInRE::isRecursiveGroup(int grid, FWObject *obj)
         {
             if (o->getId()==grid || obj->getId()==o->getId())
             {
-                compiler->abort("Group '"+o->getName()+"' references itself recursively");
+                compiler->abort(
+                    
+                        "Group '" + o->getName() + "' references itself recursively");
             }
             isRecursiveGroup(grid,o);
             isRecursiveGroup(o->getId(),o);
@@ -841,7 +890,7 @@ bool Compiler::recursiveGroupsInRE::processNext()
     Rule *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
     RuleElement *re = RuleElement::cast(rule->getFirstByType(re_type));
 
-    if (re->isAny())
+    if (re == NULL || re->isAny())
     {
         tmp_queue.push_back(rule);
         return true;
@@ -895,7 +944,7 @@ bool Compiler::emptyGroupsInRE::processNext()
     Rule *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
     RuleElement *re = RuleElement::cast(rule->getFirstByType(re_type));
 
-    if (re->isAny())
+    if (re == NULL || re->isAny())
     {
         tmp_queue.push_back(rule);
         return true;
@@ -922,10 +971,9 @@ bool Compiler::emptyGroupsInRE::processNext()
                 ostringstream  str;
                 str << "Empty group or address table object '"
                     << o->getName()
-                    << "' used in the rule "
-                    << rule->getLabel();
+                    << "'";
                 re->removeRef(o);
-                compiler->warning( str.str() );
+                compiler->warning(rule, str.str());
             }
             if (re->isAny())
             {
@@ -938,7 +986,7 @@ bool Compiler::emptyGroupsInRE::processNext()
                     << "Dropping rule "
                     <<  rule->getLabel()
                     << " because option 'Ignore rules with empty groups' is in effect";
-                compiler->warning( str.str() );
+                compiler->warning(rule,  str.str());
 
                 return true; // dropping this rule
             }
@@ -961,10 +1009,9 @@ bool Compiler::emptyGroupsInRE::processNext()
                 << sfx
                 << " '"
                 << gr
-                << "' found in the rule "
-                << rule->getLabel()
+                << "'"
                 << " and option 'Ignore rules with empty groups' is off";
-            compiler->abort( str.str() );
+            compiler->abort(rule, str.str());
         }
     }
     tmp_queue.push_back(rule);
@@ -981,7 +1028,12 @@ bool Compiler::swapMultiAddressObjectsInRE::processNext()
     Rule *rule=prev_processor->getNextRule(); if (rule==NULL) return false;
 
     RuleElement *re=RuleElement::cast( rule->getFirstByType(re_type) );
-
+    if (re == NULL || re->isAny())
+    {
+        tmp_queue.push_back(rule);
+        return true;
+    }
+    
     list<MultiAddress*> cl;
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
     {
@@ -1020,7 +1072,6 @@ bool Compiler::swapMultiAddressObjectsInRE::processNext()
                 mart->setId( mart_id );
                 compiler->dbcopy->addToIndex(mart);
                 compiler->dbcopy->add(mart);
-                compiler->cacheObj(mart);
             }
             re->removeRef(ma);
             re->addRef(mart);
@@ -1033,6 +1084,109 @@ bool Compiler::swapMultiAddressObjectsInRE::processNext()
     return true;
 }
 
+bool Compiler::expandMultipleAddressesInRE::processNext()
+{
+    Rule *rule = prev_processor->getNextRule(); if (rule==NULL) return false;
+    RuleElement *re = RuleElement::cast( rule->getFirstByType(re_type) );
+    if (re) compiler->_expand_addr(rule, re, true);
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+
+bool Compiler::checkForObjectsWithErrors::processNext()
+{
+    Rule *rule = prev_processor->getNextRule(); if (rule==NULL) return false;
+    
+    for (FWObject::iterator it1=rule->begin(); it1!=rule->end(); it1++)
+    {
+        RuleElement *re = RuleElement::cast(*it1);
+        if (re == NULL || re->isAny()) continue;
+        for (FWObject::iterator it2=re->begin(); it2!=re->end(); it2++)
+        {
+            FWObject *obj = FWReference::getObject(*it2);
+            if (obj->getBool(".rule_error"))
+            {
+                // it is ok to call abort this late in rule
+                // processing.  If the error was fatal, the code that
+                // encounter it should have called abort() then. If it
+                // continued, then we are in test mode and this call to
+                // abort will continue too. In the worst case, we end up
+                // with duplicate error messages in the test mode.
+                compiler->abort(rule, obj->getStr(".error_msg"));
+            }
+        }
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+
+bool Compiler::replaceFailoverInterfaceInRE::processNext()
+{
+    Rule *rule = prev_processor->getNextRule(); if (rule==NULL) return false;
+    RuleElement *re = RuleElement::cast( rule->getFirstByType(re_type) );
+
+    if (re == NULL || re->isAny())
+    {
+        tmp_queue.push_back(rule);
+        return true;
+    }
+
+    // list of pointers to cluster interfaces used in the RE
+    list<Interface*> cl;
+    for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
+    {
+        Interface *intf = Interface::cast(FWReference::getObject(*i));
+        if (intf==NULL) continue;
+
+        if (intf->isFailoverInterface()) cl.push_back(intf);
+        else
+        {
+            // this could be a copy of cluster interface which belongs
+            // to the firewall. This is done in
+            // Compiler::processFailoverGroup. Dont use interface name
+            // to distinguish cluster interface. Better method is to
+            // check for the variable "cluster_interface".
+            if (intf->getOptionsObject()->getBool("cluster_interface"))
+                cl.push_back(intf);
+        }
+    }
+
+    if (!cl.empty())
+    {
+        for (list<Interface*>::iterator i=cl.begin(); i!=cl.end(); i++)
+        {
+            Interface *intf = *i;
+
+            FailoverClusterGroup *fg = FailoverClusterGroup::cast(
+                intf->getFirstByType(FailoverClusterGroup::TYPENAME));
+            if (fg)
+            {
+                Interface *other_interface = fg->getInterfaceForMemberFirewall(compiler->fw);
+                re->removeRef(intf);
+                re->addRef(other_interface);
+            } else 
+            {
+                string base_interface_id = intf->getOptionsObject()->getStr("base_interface_id");
+                if (!base_interface_id.empty())
+                {
+                    FWObject *base_interface = compiler->dbcopy->findInIndex(
+                        FWObjectDatabase::getIntId(base_interface_id));
+                    if (base_interface)
+                    {
+                        re->removeRef(intf);
+                        re->addRef(base_interface);
+                    }
+                }
+            }
+        }
+    }
+    tmp_queue.push_back(rule);
+    return true;
+}
+
 bool Compiler::FindAddressFamilyInRE(FWObject *parent, bool ipv6)
 {
     Address *addr = Address::cast(parent);
@@ -1119,14 +1273,13 @@ bool Compiler::catchUnnumberedIfaceInRE(RuleElement *re)
         FWObject *o = FWReference::getObject(*i);
         if (o==NULL)
         {
-            Rule *rule = Rule::cast(re->getParent());
+            //Rule *rule = Rule::cast(re->getParent());
             FWReference *refo = FWReference::cast(*i);
             string errmsg =
                 string("catchUnnumberedIfaceInRE: Can't find object ") +
                 string("in cache, ID=") +
-                FWObjectDatabase::getStringId(refo->getPointerId()) +
-                "  rule " + rule->getLabel();
-            abort(errmsg);
+                FWObjectDatabase::getStringId(refo->getPointerId());
+            abort(re->getParent(), errmsg);
         }
         err |= ((iface=Interface::cast(o))!=NULL &&
                 (iface->isUnnumbered() || iface->isBridgePort())
@@ -1175,6 +1328,7 @@ Interface* Compiler::getFirstItf(PolicyRule *rule)
 Address* Compiler::getFirstOSrc(NATRule *rule)
 {
     RuleElementOSrc *osrc = rule->getOSrc();
+    assert(osrc!=NULL);
     FWObject *o = FWReference::getObject(osrc->front());
     return Address::cast(o);
 }
@@ -1182,6 +1336,7 @@ Address* Compiler::getFirstOSrc(NATRule *rule)
 Address* Compiler::getFirstODst(NATRule *rule)
 {
     RuleElementODst *odst = rule->getODst();
+    assert(odst!=NULL);
     FWObject *o = FWReference::getObject(odst->front());
     return Address::cast(o);
 }
@@ -1189,6 +1344,7 @@ Address* Compiler::getFirstODst(NATRule *rule)
 Service* Compiler::getFirstOSrv(NATRule *rule)
 {
     RuleElementOSrv *osrv = rule->getOSrv();
+    assert(osrv!=NULL);
     FWObject *o = FWReference::getObject(osrv->front());
     return Service::cast(o);
 }
@@ -1196,6 +1352,7 @@ Service* Compiler::getFirstOSrv(NATRule *rule)
 Address* Compiler::getFirstTSrc(NATRule *rule)
 {
     RuleElementTSrc *tsrc = rule->getTSrc();
+    assert(tsrc!=NULL);
     FWObject *o = FWReference::getObject(tsrc->front());
     return Address::cast(o);
 }
@@ -1203,6 +1360,7 @@ Address* Compiler::getFirstTSrc(NATRule *rule)
 Address* Compiler::getFirstTDst(NATRule *rule)
 {
     RuleElementTDst *tdst = rule->getTDst();
+    assert(tdst!=NULL);
     FWObject *o = FWReference::getObject(tdst->front());
     return Address::cast(o);
 }
@@ -1210,7 +1368,92 @@ Address* Compiler::getFirstTDst(NATRule *rule)
 Service* Compiler::getFirstTSrv(NATRule *rule)
 {
     RuleElementTSrv *tsrv = rule->getTSrv();
+    assert(tsrv!=NULL);
     FWObject *o = FWReference::getObject(tsrv->front());
     return Service::cast(o);
 }
 
+/*
+ * Compares given object with firewall or its parent cluster (if any).
+ * Compares only IDs of these objects. Relies on class CompilerDriver
+ * to set integer variable "parent_cluster_id" in the firewall object
+ * if it is a member of a cluster.
+ */
+bool Compiler::isFirewallOrCluster(FWObject *obj)
+{
+    int fw_id = fw->getId();
+    int cluster_id = fw->getInt("parent_cluster_id");
+    return obj->getId() == fw_id || obj->getId() == cluster_id;
+}
+
+string Compiler::printComment(Rule *rule, string &prev_rule_label,
+                              const std::string &prefix, bool suppress_comment)
+{
+    ostringstream res;
+    string rl = rule->getLabel();
+    if (rl != prev_rule_label)
+    {
+        if ( ! inSingleRuleCompileMode())
+        {
+            res << prefix << " " << endl;
+            res << prefix << " Rule  " << rl << endl;
+        }
+        string comm = rule->getComment();
+        if ( ! suppress_comment && ! comm.empty())
+        {
+            string::size_type c1, c2;
+            c1 = 0;
+            while ( (c2 = comm.find('\n', c1)) != string::npos )
+            {
+                res << prefix << " " << comm.substr(c1, c2 - c1) << endl;
+                c1 = c2 + 1;
+            }
+            string remainder =  comm.substr(c1);
+            if (!remainder.empty())
+                res << prefix << " " << remainder << endl;
+        }
+        prev_rule_label = rl;
+    }
+    string err = rule->getStr(".error_msg");
+    if (!err.empty()) res << prefix << " " << err << endl;
+    return res.str();
+}
+
+Address* Compiler::correctForCluster(Address *addr)
+{
+#ifdef DEBUG_FOR_DMZ
+    cerr << "Compiler::correctForCluster " << addr << endl;
+    if (addr)
+    {
+        cerr << "  addr: " << addr->getName() << endl;
+    }
+#endif
+
+    Interface *intf = Interface::cast(addr);
+
+    if (intf)
+    {
+#ifdef DEBUG_FOR_DMZ
+        cerr << "    intf: " << intf->getName()
+             << "    isFailoverInterface: " << intf->isFailoverInterface()
+             << endl;
+#endif
+    }
+
+    if (intf && intf->isFailoverInterface())
+    {
+        FailoverClusterGroup *fg = FailoverClusterGroup::cast(
+            intf->getFirstByType(FailoverClusterGroup::TYPENAME));
+        if (fg)
+        {
+            Address *other_intf = fg->getInterfaceForMemberFirewall(fw);
+#ifdef DEBUG_FOR_DMZ
+            cerr << "    fg: " << fg->getName() << endl;
+            cerr << "    other_intf: " << other_intf << endl;
+#endif
+            if (other_intf) return other_intf;
+        }
+    }
+    return addr;
+}
+
