@@ -4,9 +4,9 @@
 
                  Copyright (C) 2002 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Compiler_helpers.cpp 123 2008-07-05 17:38:52Z vadim $
+  $Id: Compiler_helpers.cpp 234 2008-12-23 06:17:53Z vadim $
 
 */
  
