@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Compiler_ops.cpp 234 2008-12-23 06:17:53Z vadim $
+  $Id: Compiler_ops.cpp 451 2009-10-22 21:06:56Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -425,6 +425,11 @@ bool Compiler::checkForShadowing(const Service &o1, const Service &o2)
 	    int drs2=static_cast<const TCPUDPService*>(&o2)->getDstRangeStart();
 	    int dre2=static_cast<const TCPUDPService*>(&o2)->getDstRangeEnd();
 
+            if (srs1==0 && sre1==0) sre1 = 65536; 
+            if (drs1==0 && dre1==0) dre1 = 65536;
+            if (srs2==0 && sre2==0) sre2 = 65536;
+            if (drs2==0 && dre2==0) dre2 = 65536;
+
 	    bool res = (srs1>=srs2 && sre1<=sre2 && drs1>=drs2 && dre1<=dre2);
             RETURN(res);
 	}
