@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: NATCompiler.cpp 404 2009-08-12 17:51:16Z vadim $
+  $Id: NATCompiler.cpp 598 2010-04-09 02:23:30Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -52,7 +52,7 @@ int NATCompiler::prolog()
 {
     Compiler::prolog();
 
-    FWObject  *nat=fw->getFirstByType(NAT::TYPENAME);
+    NAT *nat = NAT::cast(fw->getFirstByType(NAT::TYPENAME));
     assert(nat);
 
     combined_ruleset = new NAT();
@@ -75,13 +75,15 @@ int NATCompiler::prolog()
 //    list<FWObject*> l3=nat->getByType(NATRule::TYPENAME);
 //    for (list<FWObject*>::iterator j=l3.begin(); j!=l3.end(); ++j) {
 
-    FWObject *ruleset = source_ruleset;
+    RuleSet *ruleset = source_ruleset;
     if (ruleset == NULL)
     {
         source_ruleset = RuleSet::cast(nat);
         ruleset = nat;
     }
 
+    ruleset->renumberRules();
+
     combined_ruleset->setName(ruleset->getName());
     temp_ruleset->setName(ruleset->getName());
 
@@ -92,7 +94,19 @@ int NATCompiler::prolog()
     {
 
 	Rule *r= Rule::cast(*i);
-	if (r->isDisabled()) continue;
+        if (r == NULL) continue; // skip RuleSetOptions object
+
+        /*
+         * do not remove disabled rules just yet because some
+         * compilers might use RuleSet::insertRuleAtTop() and other
+         * similar methods from prolog() or
+         * addPredefinedPolicyRules()() and these methods renumber
+         * rules (labels stop matching rule positions when this is
+         * done because labels are configured in prolog() method of
+         * the base class. See fwbuilder ticket 1173)
+         */
+	//if (r->isDisabled()) continue;
+
 	r->setInterfaceId(-1);
 	r->setLabel( createRuleLabel(label_prefix, "NAT", r->getPosition()) );
 	r->setAbsRuleNumber(global_num); global_num++;
@@ -163,15 +177,37 @@ bool NATCompiler::classifyNATRule::processNext()
 
     if (rule->getRuleType()!=NATRule::Unknown) return true;
 
-    RuleElementTDst *tdstre=rule->getTDst();
+    RuleElementTSrc *tsrcre = rule->getTSrc();
+    RuleElementTDst *tdstre = rule->getTDst();
+    RuleElementTSrv *tsrvre = rule->getTSrv();
 
-//    Address  *osrc=compiler->getFirstOSrc(rule);
-//    Address  *odst=compiler->getFirstODst(rule);
-    Service  *osrv=compiler->getFirstOSrv(rule);
+    Service  *osrv = compiler->getFirstOSrv(rule);
 
-    Address  *tsrc=compiler->getFirstTSrc(rule);
-    Address  *tdst=compiler->getFirstTDst(rule);
-    Service  *tsrv=compiler->getFirstTSrv(rule);
+    Address  *tsrc = compiler->getFirstTSrc(rule);
+    Address  *tdst = compiler->getFirstTDst(rule);
+    Service  *tsrv = compiler->getFirstTSrv(rule);
+
+    if (rule->getAction() == NATRule::Branch)
+    {
+	rule->setRuleType(NATRule::NATBranch);
+        if (!tsrcre->isAny() || !tdstre->isAny() || !tsrvre->isAny())
+        {
+            tsrcre->clearChildren();
+            tsrcre->setAnyElement();
+
+            tdstre->clearChildren();
+            tdstre->setAnyElement();
+
+            tsrvre->clearChildren();
+            tsrvre->setAnyElement();
+
+            compiler->warning(
+                    rule,
+                    "Translated Src, Dst and Srv are ignored in the NAT "
+                    "rule with action 'Branch'");
+        }
+        return true;
+    }
 
     if (tsrc->isAny() && tdst->isAny() && tsrv->isAny())
     {
@@ -213,9 +249,11 @@ bool NATCompiler::classifyNATRule::processNext()
     if (!osrv->isAny() && !tsrv->isAny() && !( *osrv == *tsrv ) )  // have operator==, but do not have operator!=
     {
         if (osrv->getTypeName() != tsrv->getTypeName())
-            compiler->abort("NAT rule can not change service types: " +
-                              osrv->getTypeName() + " to " +
-                              tsrv->getTypeName() + "; NAT rule: "+rule->getLabel());
+            compiler->abort(
+                                rule,
+                                "NAT rule can not change service types: " +
+                                osrv->getTypeName() + " to " +
+                                tsrv->getTypeName());
 
 //      rule->setRuleType(NATRule::DNAT);
 //	return true;
@@ -288,7 +326,7 @@ bool NATCompiler::classifyNATRule::processNext()
 	return true;
     }
 
-    compiler->abort("Unsupported NAT rule: " + rule->getLabel());
+    compiler->abort(rule, "Unsupported NAT rule");
     return false;
 }
 
@@ -302,28 +340,28 @@ bool NATCompiler::ExpandMultipleAddresses::processNext()
     if (rule->getRuleType()==NATRule::NONAT ||
         rule->getRuleType()==NATRule::Return)
     {
-	rel=rule->getOSrc();    assert(rel); compiler->_expandAddr(rule,rel);
-	rel=rule->getODst();    assert(rel); compiler->_expandAddr(rule,rel);
+	rel=rule->getOSrc();    assert(rel); compiler->_expand_addr(rule, rel, true);
+	rel=rule->getODst();    assert(rel); compiler->_expand_addr(rule, rel, true);
     }
     if (rule->getRuleType()==NATRule::SNAT)
     {
-	rel=rule->getOSrc();    assert(rel); compiler->_expandAddr(rule,rel);
-	rel=rule->getODst();    assert(rel); compiler->_expandAddr(rule,rel);
-	rel=rule->getTSrc();    assert(rel); compiler->_expandAddr(rule,rel);
-	rel=rule->getTDst();    assert(rel); compiler->_expandAddr(rule,rel);
+	rel=rule->getOSrc();    assert(rel); compiler->_expand_addr(rule, rel, true);
+	rel=rule->getODst();    assert(rel); compiler->_expand_addr(rule, rel, true);
+	rel=rule->getTSrc();    assert(rel); compiler->_expand_addr(rule, rel, false);
+	rel=rule->getTDst();    assert(rel); compiler->_expand_addr(rule, rel, false);
     }
     if (rule->getRuleType()==NATRule::DNAT)
     {
-	rel=rule->getOSrc();    assert(rel); compiler->_expandAddr(rule,rel);
-	rel=rule->getODst();    assert(rel); compiler->_expandAddr(rule,rel);
-	rel=rule->getTSrc();    assert(rel); compiler->_expandAddr(rule,rel);
-	rel=rule->getTDst();    assert(rel); compiler->_expandAddr(rule,rel);
+	rel=rule->getOSrc();    assert(rel); compiler->_expand_addr(rule, rel, true);
+	rel=rule->getODst();    assert(rel); compiler->_expand_addr(rule, rel, false);
+	rel=rule->getTSrc();    assert(rel); compiler->_expand_addr(rule, rel, false);
+	rel=rule->getTDst();    assert(rel); compiler->_expand_addr(rule, rel, false);
     }
     if (rule->getRuleType()==NATRule::Redirect)
     {
-	rel=rule->getOSrc();    assert(rel); compiler->_expandAddr(rule,rel);
-	rel=rule->getODst();    assert(rel); compiler->_expandAddr(rule,rel);
-	rel=rule->getTSrc();    assert(rel); compiler->_expandAddr(rule,rel);
+	rel=rule->getOSrc();    assert(rel); compiler->_expand_addr(rule, rel, true);
+	rel=rule->getODst();    assert(rel); compiler->_expand_addr(rule, rel, false);
+	rel=rule->getTSrc();    assert(rel); compiler->_expand_addr(rule, rel, false);
     }
     return true;
 }
@@ -382,7 +420,8 @@ bool NATCompiler::checkForUnnumbered::processNext()
          compiler->catchUnnumberedIfaceInRE( rule->getODst() ) ||
          compiler->catchUnnumberedIfaceInRE( rule->getTSrc() ) ||
          compiler->catchUnnumberedIfaceInRE( rule->getTDst() ) )
-        compiler->abort("Can not use unnumbered interfaces in rules. Rule "+rule->getLabel());
+        compiler->abort(
+                            rule, "Can not use unnumbered interfaces in rules. ");
 
     tmp_queue.push_back(rule);
     return true;
@@ -412,15 +451,15 @@ bool NATCompiler::ConvertToAtomicForOriginal::processNext()
 
                 s=r->getOSrc();	assert(s);
                 s->clearChildren();
-                s->add( *i1 );
+                s->addCopyOf( *i1 );
 
                 s=r->getODst();	assert(s);
                 s->clearChildren();
-                s->add( *i2 );
+                s->addCopyOf( *i2 );
 
                 s=r->getOSrv();	assert(s);
                 s->clearChildren();
-                s->add( *i3 );
+                s->addCopyOf( *i3 );
 
                 tmp_queue.push_back(r);
 	    }
@@ -460,22 +499,22 @@ bool NATCompiler::ConvertToAtomicForAddresses::processNext()
 
 		    s=r->getOSrc();	assert(s);
 		    s->clearChildren();
-		    s->add( *i1 );
+		    s->addCopyOf( *i1 );
 
 		    s=r->getODst();	assert(s);
 		    s->clearChildren();
-		    s->add( *i2 );
+		    s->addCopyOf( *i2 );
 
 //		    s=r->getOSrv();	assert(s);
 //		    *s=*osrv;
 
 		    s=r->getTSrc();	assert(s);
 		    s->clearChildren();
-		    s->add( *i4 );
+		    s->addCopyOf( *i4 );
 
 		    s=r->getTDst();	assert(s);
 		    s->clearChildren();
-		    s->add( *i5 );
+		    s->addCopyOf( *i5 );
 
 //		    s=r->getTSrv();	assert(s);
 //		    *s=*tsrv;
@@ -506,7 +545,7 @@ bool NATCompiler::ConvertToAtomicForTSrc::processNext()
 
         s=r->getTSrc();	assert(s);
         s->clearChildren();
-        s->add( *i1 );
+        s->addCopyOf( *i1 );
 
         tmp_queue.push_back(r);
     }
@@ -531,7 +570,7 @@ bool NATCompiler::ConvertToAtomicForTDst::processNext()
 
         s=r->getTDst();	assert(s);
         s->clearChildren();
-        s->add( *i1 );
+        s->addCopyOf( *i1 );
 
         tmp_queue.push_back(r);
     }
@@ -556,7 +595,7 @@ bool NATCompiler::ConvertToAtomicForTSrv::processNext()
 
         s=r->getTSrv();	assert(s);
         s->clearChildren();
-        s->add( *i1 );
+        s->addCopyOf( *i1 );
 
         tmp_queue.push_back(r);
     }
@@ -599,28 +638,28 @@ bool NATCompiler::ConvertToAtomic::processNext()
 
 			    s=r->getOSrc();	assert(s);
 			    s->clearChildren();
-			    s->add( *i1 );
+			    s->addCopyOf( *i1 );
 
 			    s=r->getODst();	assert(s);
 			    s->clearChildren();
-			    s->add( *i2 );
+			    s->addCopyOf( *i2 );
 
 			    s=r->getOSrv();	assert(s);
 			    s->clearChildren();
-			    s->add( *i3 );
+			    s->addCopyOf( *i3 );
 
 
 			    s=r->getTSrc();	assert(s);
 			    s->clearChildren();
-			    s->add( *i4 );
+			    s->addCopyOf( *i4 );
 
 			    s=r->getTDst();	assert(s);
 			    s->clearChildren();
-			    s->add( *i5 );
+			    s->addCopyOf( *i5 );
 
 			    s=r->getTSrv();	assert(s);
 			    s->clearChildren();
-			    s->add( *i6 );
+			    s->addCopyOf( *i6 );
 
 			    tmp_queue.push_back(r);
 
@@ -664,10 +703,14 @@ bool NATCompiler::MACFiltering::processNext()
     if ( ! checkRuleElement(osrc) )
     {
         if (last_rule_lbl!=lbl)
-            compiler->warning( "MAC address matching is not supported. One or several MAC addresses removed from Original Source in the rule "+lbl);
+            compiler->warning(
+                
+                    rule, "MAC address matching is not supported. One or several MAC addresses removed from Original Source ");
 
         if (osrc->empty() || osrc->isAny())
-            compiler->abort("Original Source becomes 'Any' after all MAC addresses have been removed in the rule "+lbl);
+            compiler->abort(
+                
+                    rule, "Original Source becomes 'Any' after all MAC addresses have been removed");
 
         last_rule_lbl=lbl;
     }
@@ -676,10 +719,14 @@ bool NATCompiler::MACFiltering::processNext()
     if ( ! checkRuleElement(odst) )
     {
         if (last_rule_lbl!=lbl)
-            compiler->warning("MAC address matching is not supported. One or several MAC addresses removed from Original Destination in the rule "+lbl);
+            compiler->warning(
+                
+                    rule, "MAC address matching is not supported. One or several MAC addresses removed from Original Destination ");
 
         if (odst->empty() || odst->isAny())
-            compiler->abort("Original Destination becomes 'Any' after all MAC addresses have been removed in the rule "+lbl);
+            compiler->abort(
+                
+                    rule, "Original Destination becomes 'Any' after all MAC addresses have been removed");
 
         last_rule_lbl=lbl;
     }
@@ -805,8 +852,8 @@ string NATCompiler::debugPrintRule(libfwbuilder::Rule *r)
     FWObject::iterator i6 = tsrvrel->begin();
 
     while ( i1!=osrcrel->end() || i2!=odstrel->end() || i3!=osrvrel->end() ||
-            i4!=tsrcrel->end() || i5!=tdstrel->end() || i6!=tsrvrel->end() ) {
-
+            i4!=tsrcrel->end() || i5!=tdstrel->end() || i6!=tsrvrel->end() )
+    {
         str  << endl;
 
         string osrc = " ";
@@ -825,44 +872,51 @@ string NATCompiler::debugPrintRule(libfwbuilder::Rule *r)
         int tdst_id = -1;
         int tsrv_id = -1;
 
-        if (i1!=osrcrel->end()) {
+        if (i1!=osrcrel->end())
+        {
             FWObject *o = FWReference::getObject(*i1);
             osrc=o->getName();
             osrc_id=o->getId();
         }
 
-        if (i2!=odstrel->end()) {
+        if (i2!=odstrel->end())
+        {
             FWObject *o = FWReference::getObject(*i2);
             odst=o->getName();
             odst_id=o->getId();
         }
 
-        if (i3!=osrvrel->end()) {
+        if (i3!=osrvrel->end())
+        {
             FWObject *o = FWReference::getObject(*i3);
             osrv=o->getName();
             osrv_id=o->getId();
         }
 
-        if (i4!=tsrcrel->end()) {
+        if (i4!=tsrcrel->end())
+        {
             FWObject *o = FWReference::getObject(*i4);
             tsrc=o->getName();
             tsrc_id=o->getId();
         }
 
-        if (i5!=tdstrel->end()) {
+        if (i5!=tdstrel->end())
+        {
             FWObject *o = FWReference::getObject(*i5);
             tdst=o->getName();
             tdst_id=o->getId();
         }
 
-        if (i6!=tsrvrel->end()) {
+        if (i6!=tsrvrel->end())
+        {
             FWObject *o = FWReference::getObject(*i6);
             tsrv=o->getName();
             tsrv_id=o->getId();
         }
 
         int w=0;
-        if (no==0) {
+        if (no==0)
+        {
             str << rule->getLabel();
             w=rule->getLabel().length();
         }
