@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: OSConfigurator.cpp 234 2008-12-23 06:17:53Z vadim $
+  $Id: OSConfigurator.cpp 432 2009-09-23 16:22:20Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -33,8 +33,8 @@ using namespace std;
 OSConfigurator::~OSConfigurator() {}
 
 OSConfigurator::OSConfigurator(FWObjectDatabase *_db,
-                               const string &fwname, bool ipv6_policy) :
-    Compiler(_db, fwname, ipv6_policy)
+                               Firewall *fw, bool ipv6_policy) :
+    Compiler(_db, fw, ipv6_policy)
 {
     num_virtual_addresses_for_nat = 0;
 };
