@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: PolicyCompiler.cpp 429 2009-09-18 18:02:07Z vadim $
+  $Id: PolicyCompiler.cpp 612 2010-06-08 20:43:20Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -32,6 +32,7 @@
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/RuleElement.h"
 #include "fwbuilder/Network.h"
+#include "fwbuilder/NetworkIPv6.h"
 #include "fwbuilder/IPService.h"
 #include "fwbuilder/ICMPService.h"
 #include "fwbuilder/TCPService.h"
@@ -50,6 +51,7 @@
 #include "fwbuilder/FWException.h"
 #include "fwbuilder/Group.h"
 #include "fwbuilder/MultiAddress.h"
+#include "fwbuilder/FailoverClusterGroup.h"
 
 #include <iostream>
 #include <iomanip>
@@ -64,7 +66,7 @@ int PolicyCompiler::prolog()
 {
     Compiler::prolog();
 
-    FWObject *policy = fw->getFirstByType(Policy::TYPENAME);
+    Policy *policy = Policy::cast(fw->getFirstByType(Policy::TYPENAME));
     assert(policy);
 
     combined_ruleset = new Policy();   // combined ruleset (all interface policies and global policy)
@@ -75,12 +77,13 @@ int PolicyCompiler::prolog()
 
     int global_num=0;
 
-    FWObject *ruleset = source_ruleset;
+    RuleSet *ruleset = source_ruleset;
     if (ruleset == NULL)
     {
         source_ruleset = RuleSet::cast(policy);
         ruleset = policy;
     }
+    ruleset->renumberRules();
 
     combined_ruleset->setName(ruleset->getName());
     temp_ruleset->setName(ruleset->getName());
@@ -90,10 +93,22 @@ int PolicyCompiler::prolog()
 
     for (FWObject::iterator i=ruleset->begin(); i!=ruleset->end(); i++)
     {
-	PolicyRule *r= PolicyRule::cast(*i);
-	if (r->isDisabled()) continue;
+	PolicyRule *r = PolicyRule::cast(*i);
+        if (r == NULL) continue; // skip RuleSetOptions object
 
-        RuleElementItf *itfre=r->getItf();   assert(itfre);
+        /*
+         * do not remove disabled rules just yet because some
+         * compilers might use RuleSet::insertRuleAtTop() and other
+         * similar methods from prolog() or
+         * addPredefinedPolicyRules()() and these methods renumber
+         * rules (labels stop matching rule positions when this is
+         * done because labels are configured in prolog() method of
+         * the base class. See fwbuilder ticket 1173)
+         */
+	//if (r->isDisabled()) continue;
+
+        RuleElementItf *itfre = r->getItf();
+        assert(itfre);
 
         if (itfre->isAny())
         {
@@ -271,6 +286,27 @@ bool PolicyCompiler::checkForShadowing(PolicyRule &r1, PolicyRule &r2)
     return false;
 }
 
+/**
+ * compare interfaces of rules r1 and r2.
+ *
+ * Return true if r2 shadows r1 (only inetrface rule element is
+ * checked)
+ *
+ * If interface element is "all" (empty), it shadows any specific
+ * interface in the other rule, also "all" shadows "all". If neither
+ * is "all", return true if both rules refer the same interface,
+ * otherwise return false.
+ */
+bool PolicyCompiler::checkInterfacesForShadowing(PolicyRule &r1, PolicyRule &r2)
+{
+    int intf1_id = r1.getInterfaceId();
+    int intf2_id = r2.getInterfaceId();
+
+    if (intf2_id == -1) return true;  // "eth0" -- "all" or "all" -- "all"
+    return (intf1_id == intf2_id);
+}
+
+
 bool PolicyCompiler::cmpRules(PolicyRule &r1, PolicyRule &r2)
 {
     if (r1.getSrc()->getNeg()!=r2.getSrc()->getNeg()) return false;
@@ -309,39 +345,111 @@ bool PolicyCompiler::cmpRules(PolicyRule &r1, PolicyRule &r2)
     return ( (*src1 == *src2) && (*dst1 == *dst2) && (*srv1 == *srv2) );
 }
 
+/*
+ * Call this processor before ItfNegation (and in some compilers,
+ * singleItfNegation). 
+ */
+bool PolicyCompiler::replaceClusterInterfaceInItf::processNext()
+{
+    PolicyRule *rule=getNext(); if (rule==NULL) return false;
+
+    RuleElementItf *itfre = rule->getItf();
+    if (itfre==NULL)
+        compiler->abort(rule, "Missing interface rule element");
+
+    map<FWObject*, FWObject*> interface_replacement;
+
+    for (FWObject::iterator i=itfre->begin(); i!=itfre->end(); ++i)
+    {
+        Interface *member_iface = NULL;
+
+        // Only interface objects are allowed in the "Interface" rule element
+        FWObject *o = FWReference::getObject(*i);
+        Interface *rule_iface = Interface::cast(o);
+        if (rule_iface == NULL) continue;
+        // If this interface belongs to a cluster (which can only happen
+        // if the rule set belongs to  a cluster), then replace it with
+        // corresponding interface of the member
+        if (rule_iface->isFailoverInterface())
+        {
+            FailoverClusterGroup *fg = FailoverClusterGroup::cast(
+                rule_iface->getFirstByType(FailoverClusterGroup::TYPENAME));
+            if (fg)
+                member_iface = fg->getInterfaceForMemberFirewall(compiler->fw);
+        }
+        if (member_iface)
+        {
+            interface_replacement[rule_iface] = member_iface;
+        }
+    }
+
+    map<FWObject*, FWObject*>::iterator r;
+    for (r = interface_replacement.begin(); r != interface_replacement.end(); ++r)
+    {
+        itfre->removeRef(r->first);
+        itfre->addRef(r->second);
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
+/*
+ * Process negation in the "Interface" rule element. Scan objects in
+ * this RE, replace cluster interfaces with interfaces of the member,
+ * then replace them with a list of all other interfaces of the member.
+ *
+ * Note that normally compiler should call
+ * replaceClusterInterfaceInItf before calling this processor. This
+ * means that this processor should never see cluster interfaces in
+ * the RE. However I keep the code that deals with them in place to be
+ * able to use this processor without prior call to
+ * replaceClusterInterfaceInItf if necessary.
+ */
 bool PolicyCompiler::ItfNegation::processNext()
 {
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
 
-    // Use getByTypeDeep() to pick subinterfaces (vlans and such)
-    list<FWObject*> all_interfaces = compiler->fw->getByTypeDeep(Interface::TYPENAME);
-    list<FWObject*> work_interfaces;
-
-    // skip unprotected interfaces bug #2710034 "PF Compiler in 3.0.3
-    // Unprotected Interface Bug"
-    for (FWObject::iterator i=all_interfaces.begin(); i!=all_interfaces.end(); ++i)
-    {
-        Interface *intf = Interface::cast(*i);
-        if (intf && intf->isUnprotected()) continue;
-        work_interfaces.push_back(intf);
-    }
-
     RuleElementItf *itfre = rule->getItf();
     if (itfre==NULL)
-    {
-        compiler->abort("Missing interface RE in rule '" +
-                        rule->getLabel() +
-                        "' id=" + FWObjectDatabase::getStringId(rule->getId())
-        );
-    }
+        compiler->abort(rule, "Missing interface rule element");
 
     if (itfre->getNeg())
     {
+        // Use getByTypeDeep() to pick subinterfaces (vlans and such)
+        list<FWObject*> all_interfaces = compiler->fw->getByTypeDeep(Interface::TYPENAME);
+        list<FWObject*> work_interfaces;
+
+        // skip unprotected interfaces bug #2710034 "PF Compiler in 3.0.3
+        // Unprotected Interface Bug"
+        for (FWObject::iterator i=all_interfaces.begin(); i!=all_interfaces.end(); ++i)
+        {
+            Interface *intf = Interface::cast(*i);
+            if (intf == NULL) continue;
+            if (intf->isUnprotected()) continue;
+            if (intf->isLoopback()) continue;
+            if (intf->getOptionsObject()->getBool("cluster_interface")) continue;
+            work_interfaces.push_back(intf);
+        }
+
         for (FWObject::iterator i=itfre->begin(); i!=itfre->end(); ++i)
         {
-            FWObject *o = FWReference::getObject(*i);
-            work_interfaces.remove(o);
+            // Only interface objects are allowed in the "Interface" rule element
+            Interface *rule_iface = Interface::cast(FWReference::getObject(*i));
+            if (rule_iface == NULL) continue;
+            // If this interface belongs to a cluster (which can only happen
+            // if the rule set belongs to  a cluster), then replace it with
+            // corresponding interface of the member
+            if (rule_iface->isFailoverInterface())
+            {
+                FailoverClusterGroup *fg = FailoverClusterGroup::cast(
+                    rule_iface->getFirstByType(FailoverClusterGroup::TYPENAME));
+                if (fg)
+                    rule_iface = fg->getInterfaceForMemberFirewall(compiler->fw);
+            }
+            if (rule_iface) work_interfaces.remove(rule_iface);
         }
+
         itfre->reset();
         itfre->setNeg(false);
         for (FWObject::iterator i=work_interfaces.begin(); i!=work_interfaces.end(); ++i)
@@ -363,10 +471,10 @@ bool  PolicyCompiler::InterfacePolicyRules::processNext()
         tmp_queue.push_back(rule);
         return true;
     }
+
     for (FWObject::iterator i=itfre->begin(); i!=itfre->end(); ++i)
     {
         FWObject *o = FWReference::getObject(*i);
-
         if (ObjectGroup::isA(o))
         {
             // a group in "interface" rule element. GUI checks that only
@@ -386,15 +494,29 @@ bool  PolicyCompiler::InterfacePolicyRules::processNext()
                 PolicyRule *r= compiler->dbcopy->createPolicyRule();
                 compiler->temp_ruleset->add(r);
                 r->duplicate(rule);
+                RuleElementItf *nitf = r->getItf();
+                nitf->clearChildren();
+                nitf->setAnyElement();
+                nitf->addRef(o1);
+                // Set interface ID using setInterfaceId() for
+                // backwards compatibility with older code. This
+                // should go away eventually since we have "Interface"
+                // rule element in policy rules. Rule element like
+                // this does not exist in the NAT rules, so there we
+                // need to keep using setInterfaceId()
                 r->setInterfaceId(o1->getId());
                 tmp_queue.push_back(r);
             }
         } else
         {
-
             PolicyRule *r= compiler->dbcopy->createPolicyRule();
             compiler->temp_ruleset->add(r);
             r->duplicate(rule);
+            RuleElementItf *nitf = r->getItf();
+	    nitf->clearChildren();
+	    nitf->setAnyElement();
+            nitf->addRef(o);
+            // trying to get rid of Rule::getInterfaceId Rule::getInterfaceStr
             r->setInterfaceId(o->getId());
             tmp_queue.push_back(r);
         }
@@ -420,21 +542,20 @@ bool  PolicyCompiler::ExpandGroups::processNext()
     return true;
 }
 
-
-bool  PolicyCompiler::ExpandMultipleAddressesInSRC::processNext()
+bool PolicyCompiler::expandGroupsInSrv::processNext()
 {
-    PolicyRule *rule=getNext(); if (rule==NULL) return false;
-    RuleElementSrc *src=rule->getSrc();    assert(src);
-    compiler->_expandAddr(rule,src);
+    PolicyRule *rule = getNext(); if (rule==NULL) return false;
+    RuleElementSrv *srv = rule->getSrv();
+    compiler->expandGroupsInRuleElement(srv);
     tmp_queue.push_back(rule);
     return true;
 }
 
-bool  PolicyCompiler::ExpandMultipleAddressesInDST::processNext()
+bool PolicyCompiler::expandGroupsInItf::processNext()
 {
-    PolicyRule *rule=getNext(); if (rule==NULL) return false;
-    RuleElementDst *dst=rule->getDst();    assert(dst);
-    compiler->_expandAddr(rule,dst);
+    PolicyRule *rule = getNext(); if (rule==NULL) return false;
+    RuleElementItf *itf = rule->getItf();
+    compiler->expandGroupsInRuleElement(itf);
     tmp_queue.push_back(rule);
     return true;
 }
@@ -444,8 +565,8 @@ bool  PolicyCompiler::ExpandMultipleAddresses::processNext()
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
     RuleElementSrc *src=rule->getSrc();    assert(src);
     RuleElementDst *dst=rule->getDst();    assert(dst);
-    compiler->_expandAddr(rule,src);
-    compiler->_expandAddr(rule,dst);
+    compiler->_expand_addr(rule, src, true);
+    compiler->_expand_addr(rule, dst, true);
     tmp_queue.push_back(rule);
     return true;
 }
@@ -455,8 +576,8 @@ bool  PolicyCompiler::addressRanges::processNext()
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
     RuleElementSrc *src=rule->getSrc();    assert(src);
     RuleElementDst *dst=rule->getDst();    assert(dst);
-    compiler->_expandAddressRanges(rule,src);
-    compiler->_expandAddressRanges(rule,dst);
+    compiler->_expandAddressRanges(rule, src);
+    compiler->_expandAddressRanges(rule, dst);
     tmp_queue.push_back(rule);
     return true;
 }
@@ -598,6 +719,49 @@ bool PolicyCompiler::splitServices::processNext()
     return true;
 }
 
+bool PolicyCompiler::splitIpOptions::processNext()
+{
+    PolicyRule *rule = getNext(); if (rule==NULL) return false;
+
+    RuleElementSrv *srv = rule->getSrv();
+
+    if (srv->size()==1)
+    {
+        tmp_queue.push_back(rule);
+        return true;
+    }
+
+    list<Service*> ip_services_with_options;
+
+    for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++)
+    {
+        FWObject *o = FWReference::getObject(*i);
+
+        IPService *s = IPService::cast( o );
+        if (s==NULL) continue;
+
+        if (s->hasIpOptions()) ip_services_with_options.push_back(s);
+    }
+
+    for (list<Service*>::iterator i1=ip_services_with_options.begin();
+         i1!=ip_services_with_options.end(); i1++)
+    {
+        PolicyRule *r = compiler->dbcopy->createPolicyRule();
+        compiler->temp_ruleset->add(r);
+        r->duplicate(rule);
+        RuleElementSrv *nsrv = r->getSrv();
+        nsrv->clearChildren();
+        nsrv->addRef(*i1);
+        srv->removeRef(*i1);
+        tmp_queue.push_back(r);
+    }
+
+    tmp_queue.push_back(rule);
+
+    return true;
+}
+
+
 /*
  * processor splitServices should have been called eariler, so now all
  * services in Srv are of the same type
@@ -673,6 +837,9 @@ bool PolicyCompiler::verifyCustomServices::processNext()
  * Exceptions: 
  *   - object 'any'
  *   - interface with dynamic address.
+ *
+ * In addition check for address A.B.C.D/0 which is most likely a
+ * mistake if A.B.C.D != 0.0.0.0. See #475
  */
 Address* PolicyCompiler::checkForZeroAddr::findZeroAddress(RuleElement *re)
 {
@@ -690,9 +857,10 @@ Address* PolicyCompiler::checkForZeroAddr::findZeroAddress(RuleElement *re)
         
         if (addr==NULL && o!=NULL)
             compiler->warning(
-                string("findZeroAddress: Unknown object in rule element: ") +
-                o->getName() +
-                "  type=" + o->getTypeName());
+                    re->getParent(), 
+                    string("findZeroAddress: Unknown object in rule element: ") +
+                    o->getName() +
+                    "  type=" + o->getTypeName());
 
         if (addr && addr->hasInetAddress())
         {
@@ -714,6 +882,14 @@ Address* PolicyCompiler::checkForZeroAddr::findZeroAddress(RuleElement *re)
                     a = addr;
                     break;
                 }
+                // Address A.B.C.D/0 is most likely a mistake if
+                // A.B.C.D != 0.0.0.0
+                if ((Network::cast(addr) || NetworkIPv6::cast(addr)) &&
+                    !ad->isAny() && nm!=NULL && nm->isAny())
+                {
+                    a = addr;
+                    break;
+                }
             }
         }
     }
@@ -724,22 +900,22 @@ Address* PolicyCompiler::checkForZeroAddr::findZeroAddress(RuleElement *re)
 Address* PolicyCompiler::checkForZeroAddr::findHostWithNoInterfaces(
     RuleElement *re)
 {
-    Address *a=NULL;
-
     for (FWObject::iterator i=re->begin(); i!=re->end(); i++) 
     {
         FWObject *o = FWReference::getObject(*i);
 	assert(o!=NULL);
         Host *addr = Host::cast(o);
-        // if host has child of type Interface, it must be first of the children
-        if (addr!=NULL && addr->front()!=NULL && Interface::isA(addr->front()))
+
+        if (addr!=NULL && addr->front()!=NULL)
         {
-            a=addr;
-            break;
+            FWObject::iterator it;
+            it=addr->begin();
+            while (it!=addr->end() && !Interface::isA(*it)) ++it;
+            if (it==addr->end()) return addr; // has no interfaces
         }
     }
 
-    return a;
+    return NULL;
 }
 
 
@@ -757,10 +933,10 @@ bool PolicyCompiler::checkForZeroAddr::processNext()
     if (a==NULL) a = findHostWithNoInterfaces( rule->getDst() );
 
     if (a!=NULL)
-        compiler->abort("Object '"+a->getName()+
-                        "' has no interfaces, therefore it does not have "
-                        "address and can not be used in the rule."+
-                        " Rule "+rule->getLabel());
+        compiler->abort(
+                rule, "Object '"+a->getName()+
+                "' has no interfaces, therefore it does not have "
+                "address and can not be used in the rule.");
 
     a = findZeroAddress( rule->getSrc() );
     if (a==NULL) a = findZeroAddress( rule->getDst() );
@@ -780,10 +956,10 @@ bool PolicyCompiler::checkForZeroAddr::processNext()
                 err+=" )";
             }
         }
-        err += " has address 0.0.0.0, which is equivalent to 'any'. "
-            "This is most likely an error. Rule " + rule->getLabel();
+        err += " has address or netmask 0.0.0.0, which is equivalent to 'any'. "
+            "This is likely an error.";
 
-        compiler->abort(err);
+        compiler->abort(rule, err);
     }
 
     tmp_queue.push_back(rule);
@@ -798,7 +974,9 @@ bool PolicyCompiler::checkForUnnumbered::processNext()
 
     if ( compiler->catchUnnumberedIfaceInRE( rule->getSrc() ) || 
          compiler->catchUnnumberedIfaceInRE( rule->getDst() ) )
-        compiler->abort("Can not use unnumbered interfaces in rules. Rule "+rule->getLabel());
+        compiler->abort(
+            
+                rule, "Can not use unnumbered interfaces in rules.");
 
     tmp_queue.push_back(rule);
     return true;
@@ -822,11 +1000,11 @@ bool PolicyCompiler::ConvertToAtomicForAddresses::processNext()
 	    FWObject *s;
 	    s=r->getSrc();	assert(s);
 	    s->clearChildren();
-	    s->add( *i1 );
+	    s->addCopyOf( *i1 );
 
 	    s=r->getDst();	assert(s);
 	    s->clearChildren();
-	    s->add( *i2 );
+	    s->addCopyOf( *i2 );
 
 	    tmp_queue.push_back(r);
 	}
@@ -855,7 +1033,7 @@ bool PolicyCompiler::ConvertToAtomicForIntervals::processNext()
 
         s=r->getWhen();	assert(s);
         s->clearChildren();
-        s->add( *i1 );
+        s->addCopyOf( *i1 );
 
         tmp_queue.push_back(r);
     }
@@ -882,15 +1060,15 @@ bool  PolicyCompiler::ConvertToAtomic::processNext()
                 FWObject *s;
                 s=r->getSrc();	assert(s);
                 s->clearChildren();
-                s->add( *i1 );
+                s->addCopyOf( *i1 );
 
                 s=r->getDst();	assert(s);
                 s->clearChildren();
-                s->add( *i2 );
+                s->addCopyOf( *i2 );
 
                 s=r->getSrv();	assert(s);
                 s->clearChildren();
-                s->add( *i3 );
+                s->addCopyOf( *i3 );
 
                 tmp_queue.push_back(r);
 	    }
@@ -900,6 +1078,12 @@ bool  PolicyCompiler::ConvertToAtomic::processNext()
 }
 
 
+bool PolicyCompiler::checkForShadowingPlatformSpecific(PolicyRule * /*UNUSED r1 */,
+                                                       PolicyRule * /*UNUSED r2 */)
+{
+    return true;
+}
+
 std::deque<Rule*>::iterator 
 PolicyCompiler::findMoreGeneralRule::find_more_general_rule(
     PolicyRule *rule,
@@ -921,15 +1105,21 @@ PolicyCompiler::findMoreGeneralRule::find_more_general_rule(
     {
 	PolicyRule *r = PolicyRule::cast( *j );
 
-	if (! check_interface || 
-	    (rule->getInterfaceId()==r->getInterfaceId()) ) 
+        bool intf_cr = false;
+        if (reverse)
+            intf_cr = pcomp->checkInterfacesForShadowing( *r , *rule );
+        else
+            intf_cr = pcomp->checkInterfacesForShadowing( *rule , *r );
+
+	if (! check_interface || intf_cr)
         {
             bool cr = false;
             if (reverse)
                 cr = pcomp->checkForShadowing( *r , *rule );
             else
                 cr = pcomp->checkForShadowing( *rule , *r );
-            if ( cr ) 
+
+            if ( cr && pcomp->checkForShadowingPlatformSpecific(rule, r))
             {
                 if (compiler->debug>=9) 
                 {
@@ -951,7 +1141,6 @@ PolicyCompiler::findMoreGeneralRule::find_more_general_rule(
     return j;
 }
 
-
 bool PolicyCompiler::DetectShadowing::processNext()
 {
     PolicyRule *rule;
@@ -976,8 +1165,9 @@ bool PolicyCompiler::DetectShadowing::processNext()
         if (r && r->getAbsRuleNumber() != rule->getAbsRuleNumber() && 
             ! (*r == *rule) ) 
         {
-            compiler->abort("Rule '" + r->getLabel() +
-             "' shadows rule '" + rule->getLabel() + "'  below it");
+            compiler->abort(
+                                r, "Rule '" + r->getLabel() +
+                                "' shadows rule '" + rule->getLabel() + "'  below it");
         }
     }
 
@@ -1010,8 +1200,11 @@ bool PolicyCompiler::DetectShadowingForNonTerminatingRules::processNext()
         if (r && r->getAbsRuleNumber() != rule->getAbsRuleNumber() && 
             ! (*r == *rule) ) 
         {
-            compiler->abort("Non-terminating rule '" + rule->getLabel() +
-             "' shadows rule '" + r->getLabel() + "'  above it");
+            compiler->abort(
+                
+                    rule, 
+                    "Non-terminating rule '" + rule->getLabel() +
+                    "' shadows rule '" + r->getLabel() + "'  above it");
         }
     }
 
@@ -1050,10 +1243,17 @@ bool PolicyCompiler::MACFiltering::processNext()
     if ( ! checkRuleElement(src) )
     {
         if (last_rule_lbl!=lbl)
-            compiler->warning( "MAC address matching is not supported. One or several MAC addresses removed from source in the rule "+lbl);
+            compiler->warning(
+                
+                    rule, "MAC address matching is not supported. "
+                    "One or several MAC addresses removed from source in the rule");
 
         if (src->empty() || src->isAny())
-            compiler->abort("Source becomes 'Any' after all MAC addresses have been removed in the rule "+lbl);
+            compiler->abort(
+                
+                    rule, 
+                    "Source becomes 'Any' after all MAC addresses "
+                    "have been removed in the rule");
 
         last_rule_lbl=lbl;
     }
@@ -1062,10 +1262,18 @@ bool PolicyCompiler::MACFiltering::processNext()
     if ( ! checkRuleElement(dst) )
     {
         if (last_rule_lbl!=lbl)
-            compiler->warning("MAC address matching is not supported. One or several MAC addresses removed from destination in the rule "+lbl);
+            compiler->warning(
+                
+                    rule, 
+                    "MAC address matching is not supported. "
+                    "One or several MAC addresses removed from destination in the rule");
 
         if (dst->empty() || dst->isAny())
-            compiler->abort("Destination becomes 'Any' after all MAC addresses have been removed in the rule "+lbl);
+            compiler->abort(
+                
+                    rule, 
+                    "Destination becomes 'Any' after all MAC addresses "
+                    "have been removed in the rule ");
 
         last_rule_lbl=lbl;
     }
@@ -1088,7 +1296,13 @@ bool PolicyCompiler::CheckForTCPEstablished::processNext()
         if (s==NULL) continue;
 
         if (s->getEstablished())
-            compiler->abort(string("TCPService object with option \"established\" is not supported by firewall platform \"") + compiler->myPlatformName() + string("\". Use stateful rule instead."));
+            compiler->abort(
+                
+                    rule, 
+                    string("TCPService object with option \"established\" "
+                           "is not supported by firewall platform \"") +
+                    compiler->myPlatformName() +
+                    string("\". Use stateful rule instead."));
     }
 
     tmp_queue.push_back(rule);
@@ -1106,8 +1320,11 @@ bool PolicyCompiler::CheckForUnsupportedUserService::processNext()
         FWObject *o = FWReference::getObject(*i);
 
         if (UserService::isA(o))
-            compiler->abort(string("UserService object is not supported by ") +
-                            compiler->myPlatformName());
+            compiler->abort(
+                
+                    rule, 
+                    string("UserService object is not supported by ") +
+                    compiler->myPlatformName());
     }
 
     tmp_queue.push_back(rule);
@@ -1183,9 +1400,10 @@ string PolicyCompiler::debugPrintRule(Rule *r)
     RuleElementSrc *srcrel=rule->getSrc();
     RuleElementDst *dstrel=rule->getDst();
     RuleElementSrv *srvrel=rule->getSrv();
+    RuleElementItf *itfrel=rule->getItf();
 
     int iface_id = rule->getInterfaceId();
-    Interface *rule_iface = fw_interfaces[iface_id];
+    Interface *rule_iface = Interface::cast(dbcopy->findInIndex(iface_id));
 
     ostringstream str;
 
@@ -1195,21 +1413,26 @@ string PolicyCompiler::debugPrintRule(Rule *r)
     FWObject::iterator i1=srcrel->begin();
     FWObject::iterator i2=dstrel->begin(); 
     FWObject::iterator i3=srvrel->begin();
-    while ( i1!=srcrel->end() || i2!=dstrel->end() || i3!=srvrel->end() ) {
+    FWObject::iterator i4=itfrel->begin();
 
+    while ( i1!=srcrel->end() || i2!=dstrel->end() || i3!=srvrel->end() ||
+            i4!=itfrel->end())
+    {
         str << endl;
 
         string src=" ";
         string dst=" ";
         string srv=" ";
+        string itf=" ";
 
         int src_id = -1;
         int dst_id = -1;
         int srv_id = -1;
 
-        if (srcrel->getNeg()) src="!";
-        if (dstrel->getNeg()) dst="!";
-        if (srvrel->getNeg()) srv="!";
+        if (srcrel->getNeg()) src = "!";
+        if (dstrel->getNeg()) dst = "!";
+        if (srvrel->getNeg()) srv = "!";
+        if (itfrel->getNeg()) itf = "!";
 
         if (i1!=srcrel->end())
         {
@@ -1232,6 +1455,12 @@ string PolicyCompiler::debugPrintRule(Rule *r)
             srv_id = o->getId();
         }
 
+        if (i4!=itfrel->end()) {
+            FWObject *o=*i4;
+            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+            itf+=o->getName();
+        }
+
         int w=0;
         if (no==0) {
             str << rule->getLabel();
@@ -1243,6 +1472,7 @@ string PolicyCompiler::debugPrintRule(Rule *r)
         str <<  setw(18) << setfill(' ') << src.c_str() << "(" << src_id << ")";
         str <<  setw(18) << setfill(' ') << dst.c_str() << "(" << dst_id << ")";
         str <<  setw(12) << setfill(' ') << srv.c_str() << "(" << srv_id << ")";
+        str <<  setw(8)  << setfill(' ') << itf.c_str();
 
         if (no==0)
         {
@@ -1258,7 +1488,67 @@ string PolicyCompiler::debugPrintRule(Rule *r)
         if ( i1!=srcrel->end() ) ++i1;
         if ( i2!=dstrel->end() ) ++i2;
         if ( i3!=srvrel->end() ) ++i3;
+        if ( i4!=itfrel->end() ) ++i4;
     }
     return str.str();
 }
 
+PolicyRule* PolicyCompiler::addMgmtRule(Address* src,
+                                        Address* dst,
+                                        Service* service,
+                                        Interface* iface,
+                                        const PolicyRule::Direction direction,
+                                        const PolicyRule::Action action,
+                                        const string &label)
+{
+    assert(combined_ruleset != NULL);
+
+    /* Insert PolicyRules at top so they do not get shadowed by other
+     * rules. Call insertRuleAtTop() with hidden_rule argument true to
+     * make sure this rule gets negative position number and does not
+     * shift positions of other rules. See ticket #16. Also, hidden
+     * rules are not considered for shadowing.
+     */
+
+    PolicyRule* rule = PolicyRule::cast(combined_ruleset->insertRuleAtTop(true));
+    assert(rule != NULL);
+
+    ostringstream str;
+    str << rule->getPosition() << " " << label << " (automatic)" ;
+    rule->setLabel(str.str());
+
+    FWObject *re;
+    re = rule->getSrc();  assert(re!=NULL);
+    RuleElementSrc::cast(re)->reset();
+    if(src != NULL)
+        re->addRef(src);
+
+    re = rule->getDst();  assert(re!=NULL);
+    RuleElementDst::cast(re)->reset();
+    if(dst != NULL)
+        re->addRef(dst);
+
+    re = rule->getSrv();  assert(re!=NULL);
+    RuleElementSrv::cast(re)->reset();
+    if(service != NULL)
+        re->addRef(service);
+
+    re = rule->getWhen(); assert(re!=NULL);
+    RuleElementInterval::cast(re)->reset();
+
+    re = rule->getItf(); assert(re!=NULL);
+    RuleElementItf::cast(re)->reset();
+    if(iface != NULL)
+    {
+        re->addRef(iface);
+        rule->setInterfaceId(iface->getId());
+    }
+
+    rule->add(dbcopy->create(PolicyRuleOptions::TYPENAME));
+    rule->setLogging(false);
+    rule->enable();
+    rule->setAction(action);
+    rule->setDirection(direction);
+    
+    return rule;
+}
